﻿
// ==========================================================================================================
//  Glideslope 2 (GS2) Main code
//  Copyright (c) 2012 Andrew (ADSWNJ) Stokes, Licensed under Gnu LGPL
//
//  Designed to support flying of an accurate glide slope for the Orbiter simulator (tested for Orbiter 2010)
//
//
//	Derivitive work based on original Glideslope MFD Copyright (c) 2006, 2009 by Chris "KWAN3217" Jeppesen
//    licensed under Gnu LGPL
// 
//  The Orbiter 2010 simulator (c) Martin Schweiger
//
//                   All rights reserved
//
// Glideslope2.cpp
// ===========================================================================================================

#define STRICT
#define ORBITER_MODULE
#define _CRT_SECURE_NO_WARNINGS
#include "windows.h"
#include "orbitersdk.h"
#include "DisplayEngUnitFunctions.h"
#include "ParseFunctions.h"
#include "GlideslopeCore.h"
#include "GSMiscFunctions.h"
#include "GS2.h"

// ==============================================================
// Global variables

GlideslopeCore gg[1];
int nGutsUsed;
int g_MFDmode;


// ==============================================================
// MFD class implementation

// Constructor
Glideslope::Glideslope (DWORD w, DWORD h, VESSEL *v, UINT im): MFD (w, h, v) {
  vessel=v;
  width=w;
  height=h;
  mid=w/2;

  ZoomX=10000.0*1e3;
  ZoomY=30;
  ZoomMap=60000;
  MinZoomMap=40000;
  ManZoomToggle = false;
  MinZoomX=ZoomX/(2<<9);
  MinZoomY=ZoomY/(2<<5);;
  units=false;//False==metric, true==US
  imfd=im;


// This code is looking for a pre-initialized common guts area. The issue is that the Glideslope class gets destroyed and recreated each time
// you change view (e.g. F8), or resize an external MFD. We need to persist common things (e.g. target runway) into the guts core.
  bool found=false;
  bool mfound=false;


  for(int i=0;i<nGutsUsed;i++) {
    if(gg[i].vessel=vessel) {
      found=true;
	  G=&gg[i];

// This code looks for a specific example of this vessel and this MFD. If not found, we load it and set a base mode for it. 
	  for(int j=0;((j<G->gutsMFDcount)&&(!mfound));j++) {
		  if (G->gutsMFDint[j] == imfd) {
		    mfound = true;
			gutsMFDref = j;
			mode = G->gutsMode[j];	// get my old mode back!
		  }
	  }
	  if (!mfound) {
		  gutsMFDref = G->gutsMFDcount;
          G->gutsMFDint[gutsMFDref] = imfd;		// Create this new vessel/MFD pair and init the settings
		  if (imfd == MFD_LEFT) {
  		    G->gutsMode[gutsMFDref] = G->ParamsModeL;
		  } else if (imfd == MFD_RIGHT) {
  		    G->gutsMode[gutsMFDref] = G->ParamsModeR;
		  } else {
  		    G->gutsMode[gutsMFDref] = G->ParamsModeX;
		  }
		  mode = G->gutsMode[gutsMFDref];
		  G->gutsMFDcount++;

	  }
    }
  }
  if(!found) {
    G=&gg[nGutsUsed];
    nGutsUsed++;
    LoadConfig(); 
    G->reset(vessel);
	
    for (int i=1; i<G->gutsMFDlimit; i++) {
	  G->gutsMFDint[i] = 0;
	  G->gutsMode[i] = 0;
    }
    G->gutsMFDcount=1;
	gutsMFDref=0;
    G->gutsMFDint[gutsMFDref] = imfd;		// Create this first vessel/MFD pair and init the settings
	if (imfd == MFD_LEFT) {
  	  G->gutsMode[gutsMFDref] = G->ParamsModeL;
	} else if (imfd == MFD_RIGHT) {
  	  G->gutsMode[gutsMFDref] = G->ParamsModeR;
	} else {
  	  G->gutsMode[gutsMFDref] = G->ParamsModeX;
	}
    mode = G->gutsMode[gutsMFDref];
  }


  int oldMode=mode;
  if ((mode<1)||(mode>G->MaxMOD)) mode=1;
//sprintf(oapiDebugString(),"GS2 constructor called...  iMFD(%i) gutsinit(%c) mfdinit(%c)",imfd, (found?'Y':'N'), (mfound?'Y':'N'));

  blueBrush=CreateSolidBrush(BLUE);
  dkblueBrush=CreateSolidBrush(DARK_BLUE);
  blackBrush=CreateSolidBrush(BLACK);
  whitePen=CreatePen(PS_SOLID,1,WHITE);
  yellowPen=CreatePen(PS_SOLID,1,YELLOW);


}


// Destructor
Glideslope::~Glideslope () {
  DeleteObject(blueBrush);
  DeleteObject(dkblueBrush);
  DeleteObject(blackBrush);
  DeleteObject(whitePen);
  DeleteObject(yellowPen);
}

int Glideslope::line( int i ) {
  return (int)((float)i*((float)height/25.0));
}

int Glideslope::MapX(double x) {
  return (int)(x*width/ZoomX);
}

int Glideslope::MapY(double y) {
  return height-(int)(y*height/ZoomY);
}

int Glideslope::MapX(AZ_RANGE a) {
  double x=a.range*sin(a.az);
  return (int)(x*width/ZoomMap+width/2);
}

int Glideslope::MapY(AZ_RANGE a) {
  double y=a.range*cos(a.az);
  return (int) (height*5./6.-(int)(y*height/ZoomMap));
}

// Repaint the MFD
void Glideslope::Update (HDC hDC) {
  char buf[256];
  Title (hDC, "GS2");
  double SimT=oapiGetSimTime();

  int l=1;
  int maxx,maxy;
  VECTOR3 nre_tc_sc;
  VECTOR3 xyz_tc;
  MATRIX3 M_tc;
  VECTOR3 nre_sc_tc;
  AZ_RANGE ar_tc_sc;
  AZ_RANGE ar_sc_tc;
  AZ_RANGE ar_feature_sc;
  AZ_RANGE ar_feature_wp1;
  AZ_RANGE ar_feature_wp2;
  double turnRadius;
  int pwr = 0;
  char sym = ' ';

  switch(mode) {
    case 0:
      //Should never get here
      SetTextColor( hDC, RED );
      sprintf(buf,"Mode 0 selected in error");
      TextOut(hDC,5,line(4),buf,strlen(buf));
	  break;
//
// MFD Mode 1 ... Reference glideslope altitude and speed
//
    case 1:
      //Draw reference glideslope (altitude)
      SetTextColor( hDC, YELLOW );
      sprintf(buf,"ALT");
      TextOut(hDC,5,line(4),buf,strlen(buf));
      SetTextColor( hDC, GREEN );
      sprintf(buf,"TAS");
      TextOut(hDC,5,line(3),buf,strlen(buf));

	  //Draw Ref TAS
	  SelectDefaultPen(hDC,2);
      MoveToEx(hDC,MapX(G->RefSlope[0][2]),MapY(G->RefSlope[0][0]/300.0),NULL);
      for(int i=1;i<G->RefSlopePts;i++) {
        LineTo(hDC,MapX(G->RefSlope[i][2]),MapY(G->RefSlope[i][0]/300.0));
      }


      //Draw Ref ALT
	  SelectDefaultPen(hDC,3);
      MoveToEx(hDC,MapX(G->RefSlope[0][2]),MapY(G->RefSlope[0][3]/5000.0),NULL);
      for(int i=1;i<G->RefSlopePts;i++) {
        LineTo(hDC,MapX(G->RefSlope[i][2]),MapY(G->RefSlope[i][3]/5000.0));
      }
      //Draw tracks
      if(G->TrackPtr>0) {
        SelectDefaultPen(hDC,1);
        MoveToEx(hDC,MapX(G->TrackD[0]),MapY(G->TrackV[0]/300),NULL);
        for(int i=1;i<G->TrackPtr;i++) {
          LineTo(hDC,MapX(G->TrackD[i]),MapY(G->TrackV[i]/300));
        }
        HPEN oldPen=(HPEN)SelectObject(hDC,yellowPen);
        MoveToEx(hDC,MapX(G->TrackD[0]),MapY(G->TrackA[0]/5000.0),NULL);
        for(int i=1;i<G->TrackPtr;i++) {
          LineTo(hDC,MapX(G->TrackD[i]),MapY(G->TrackA[i]/5000.0));
        }

		SelectObject(hDC,oldPen);
      }

      //Autozoom
      maxx=MapX(G->TrackD[G->TrackPtr-1]);
  	  if(maxx>width) ZoomX*=2;
      if(maxx<width*9/20) ZoomX/=2;
      if(ZoomX<MinZoomX)ZoomX=MinZoomX;
      maxy=MapY(G->TrackA[G->TrackPtr-1]/5000);
      if(MapY(G->TrackV[G->TrackPtr-1]/300)<maxy)maxy=MapY(G->TrackV[G->TrackPtr-1]/300);
      if(maxy<0)ZoomY*=2;
      if(maxy>height*11/20)ZoomY/=2;
      if(ZoomY<MinZoomY)ZoomY=MinZoomY;
      SetTextColor( hDC, WHITE );
      sprintf(buf,"%s",G->RunwayName[G->runway]);
      TextOut(hDC,5,line(24),buf,strlen(buf));
      sprintf(buf,"%s",G->BaseName[G->RunwayBase[G->runway]]);
      TextOut(hDC,5,line(1),buf,strlen(buf));
      break;
//
// MFD Mode 2 ... Tapes output mode
//
	case 2: // TAPES OUPUT
      SelectDefaultFont(hDC,1);
      sprintf(buf,"%s",G->RunwayName[G->runway]);
      TextOut(hDC,5,line(1),buf,strlen(buf));
      DisplayEngUnit2(buf,"Range:      %5.0f%c","Range:  %9.3f%c","m","NMi",1.0/1852.0,units,G->range);
      TextOut(hDC,5,line(2),buf,strlen(buf));
      sprintf(buf,"DelAz");
      TextOut(hDC,5,line(24),buf,strlen(buf));
      //AOA scale
      VScale(hDC,0,"%4.1f","%4.1f",1.0,G->Refalpha*DEG,G->alpha*DEG,0,50,1*width/8,G->Refalpha*DEG-3,G->Refalpha*DEG+3 );
      //Airspeed scale
      VTape(hDC,"TAS","m/s","ft/s",500,100,1.0/0.3048,G->RefAirspeed,G->Airspeed,3*width/8,4*width/8-2);
      //Altitude scale
      VTape(hDC,"dTAS","m/s²","ft/s²",10,2,1.0/0.3048,G->RefAirspeedRate,G->AirspeedRate,4*width/8,5*width/8-2);
      //Altitude scale
      VTape(hDC,"ALT","m","ft",5000,1000,1.0/0.3048,G->RefAltitude,G->Altitude,5*width/8,6*width/8-2);
      //VSpd scale
      VTape(hDC,"VSPD","m/s","ft/s",50,10,1.0/0.3048,G->RefVspd,G->vspd,6*width/8,7*width/8-2);
      //VSpd Ratescale
      VTape(hDC,"VACC","m/s²","ft/s²",10,2,1.0/0.3048,G->RefVspdRate,G->VspdRate,7*width/8,8*width/8-2);
      //Energy scale
//      VTape(hDC,"ENE","J","ftlbf",5e5,1e5,0.7375621,G->RefTE,G->TE,6*width/8,7*width/8-2);
      //Energy loss
//      VTape(hDC,"E/W loss","J/s","ftlbf/s",5e3,1e3,0.7375621,fabs(G->RefTERate),fabs(G->TERate),7*width/8,8*width/8-2);
      //Delaz scale
      SelectDefaultFont(hDC,1);
      HScale(hDC,3,"%6.1f°","%6.1f°",1.0,G->DelAz*DEG,-20,20,line(24),true, -5, 5);
      SelectDefaultFont(hDC,1);
      break;

//
// MFD Mode 3 ... Digital Descent Display mode
//
    case 3: // Digital Descent Display

      SelectDefaultFont(hDC,1);
      SetTextColor( hDC, WHITE );
	  sprintf(buf, "Digital Descent Data");
      TextOut(hDC,5,line(1),buf,strlen(buf));
      sprintf(buf,"%s",G->BaseName[G->RunwayBase[G->runway]]);
      TextOut(hDC,5,line(2),buf,strlen(buf));
      sprintf(buf,"%s",G->RunwayName[G->runway]);
      TextOut(hDC,5,line(3),buf,strlen(buf));
	  l=5; 
	  sprintf(buf, "          Actual");
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, WHITE );
	  sprintf(buf, "  Reference");
      TextOut(hDC,mid,line(l),buf,strlen(buf));

	  // Delta Azimuth display
	  l++;
      if (fabs(G->DelAz*DEG)< 5.0) {
        SetTextColor( hDC, GREEN );
      } else if(G->DelAz*DEG <= -5) {
        SetTextColor( hDC, RED );
      } else {
	    SetTextColor(hDC, WHITE);
      }
	  DisplayEngUnit3(buf,"DelAz:  %7.2f",G->DelAz*DEG,&sym,&pwr,0,6);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, WHITE );
	  DisplayEngUnit(buf,"  %7.2f    °",0.0);
      TextOut(hDC,mid,line(l),buf,strlen(buf));

	  // Angle of Attack (Alpha) display
	  l++;
      if (fabs(G->alpha*DEG - G->Refalpha*DEG)< 2.1) {
        SetTextColor( hDC, GREEN );
      } else if(G->alpha*DEG < G->Refalpha*DEG) {
        SetTextColor( hDC, RED );
      } else {
	    SetTextColor(hDC, WHITE);
      }
	  DisplayEngUnit(buf,"AoA:    %7.2f",G->alpha*DEG);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, WHITE );
	  DisplayEngUnit(buf,"  %7.2f    °",G->Refalpha*DEG);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
	  

      // Range display
	  l++; l++;
	  SetTextColor( hDC, GREEN );
	  DisplayEngUnit2(buf,"Range:%8.1f%c","Range:  %8.3f%c","","",1.0/1852.0,units,G->range);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, WHITE );
	  DisplayEngUnit2(buf,"%8.1f%c","  %8.3f%c","    m","  NMi",1.0/1852.0,units,G->range);
      TextOut(hDC,mid,line(l),buf,strlen(buf));

	  // Range TO HAC WAYPOINT display
	  if (G->leg == 1) {
		l++;
		SetTextColor( hDC, GREEN );
	    DisplayEngUnit2(buf," ToHAC:%7.1f%c"," ToHAC: %8.3f%c","","",1.0/1852.0,units,G->range_wp1_sc);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        SetTextColor( hDC, WHITE );
	    DisplayEngUnit2(buf,"%8.1f%c","  %8.3f%c","    m","  NMi",1.0/1852.0,units,G->range_wp1_sc);
        TextOut(hDC,mid,line(l),buf,strlen(buf));
	  }
	
	  // Energy display
	  l++; l++;
      if (abs(G->TE - G->RefTE)< (0.1 * G->RefTE)) {
        SetTextColor( hDC, GREEN );
      } else if (G->TE < G->RefTE) {
        SetTextColor( hDC, RED );
      } else {
	    SetTextColor(hDC, WHITE);
      }
      DisplayEngUnit2(buf,"Energy:%9.3f%c","Energy:%9.3f%c","","",0.7375621,units,G->TE);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, WHITE );
	  DisplayEngUnit2(buf," %9.3f%c","%9.3f%c","  J","   ftlbf",0.7375621,units,G->RefTE);
      TextOut(hDC,mid,line(l),buf,strlen(buf));

//sprintf(oapiDebugString(),"TE(%9.6f) PE(%9.6f) KE(%9.6f) GM1(%9.6f) M2(%9.6f) re(%9.6f) Alt(%9.6f) TAS(%9.6f)", 
//	G->TE, G->PE, G->KE, G->GtimesM1, G->M2, G->re, G->Altitude, G->Airspeed);  

	  // Delta Energy display
	  l++;
      if (fabs(G->TERate - G->RefTERate)< fabs(0.5 * G->RefTERate)) {
        SetTextColor( hDC, GREEN );
      } else if (G->TERate < G->RefTERate) {
        SetTextColor( hDC, RED );
	  } else {
	    SetTextColor(hDC, WHITE);
	  }

	  DisplayEngUnit2(buf," dEngy: %6.1f%c"," dEngy: %6.1f%c","","",0.7375621,units,G->TERate);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, WHITE );
	  DisplayEngUnit2(buf,"  %6.1f%c","  %6.1f%c","    J/s","  ftlbf/s",0.7375621,units,G->RefTERate);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
	  
	  // Altitude display
	  l++; l++;
      if (fabs(G->Altitude - G->RefAltitude)< (0.1 * G->RefAltitude)) {
        SetTextColor( hDC, GREEN );
      } else if (G->Altitude < G->RefAltitude) {
        SetTextColor( hDC, RED );
      } else {
	    SetTextColor(hDC, WHITE);
      }
	  DisplayEngUnit2(buf,"Alt:    %7.2f%c","Alt:    %7.2f%c","","",1.0/0.3048,units,G->Altitude);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, WHITE );
	  DisplayEngUnit2(buf,"  %7.2f%c","  %7.2f%c","   m","   ft",1.0/0.3048,units,G->RefAltitude);
      TextOut(hDC,mid,line(l),buf,strlen(buf));


	  // Vertical Speed (Delta Altitude) display
	  l++;
      if (fabs(G->vspd - G->RefVspd)< (0.25 * abs(G->RefVspd)) || fabs(G->vspd - G->RefVspd)< 40.0) {
        SetTextColor( hDC, GREEN );
      } else if(G->vspd < G->RefVspd) {
        SetTextColor( hDC, RED );
      } else {
	    SetTextColor(hDC, WHITE);
      }
	  DisplayEngUnit2(buf," dAlt:  %7.2f%c"," dAlt:  %7.2f%c","","",1.0/0.3048,units,G->vspd);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, WHITE );
	  DisplayEngUnit2(buf,"  %7.2f%c","  %7.2f%c","   m/s","   ft/s",1.0/0.3048,units,G->RefVspd);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
	  	  
      // True Air Speed display
	  l++; l++;
      if (fabs(G->Airspeed - G->RefAirspeed)< (0.1 * G->RefAirspeed)) {
        SetTextColor( hDC, GREEN );
      } else if (G->Airspeed < G->RefAirspeed) {
        SetTextColor( hDC, RED );
      } else {
	    SetTextColor(hDC, WHITE);
      }
	  DisplayEngUnit2(buf,"TAS:    %7.2f%c","TAS:    %7.2f%c","","",1.0/0.3048,units,G->Airspeed);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, WHITE );
	  DisplayEngUnit2(buf,"  %7.2f%c","  %7.2f%c","   m/s","   ft/s",1.0/0.3048,units,G->RefAirspeed);
      TextOut(hDC,mid,line(l),buf,strlen(buf));

	  // Delta True Air Speed display
	  l++;
      if (fabs(G->AirspeedRate - G->RefAirspeedRate)< (0.25 * abs(G->RefAirspeedRate)) || fabs(G->AirspeedRate - G->RefAirspeedRate) < 20.0) {
        SetTextColor( hDC, GREEN );
      } else if (G->AirspeedRate < G->RefAirspeedRate) {
        SetTextColor( hDC, RED );
      } else {
	    SetTextColor(hDC, WHITE);
      }
	  DisplayEngUnit2(buf," dTAS:  %7.2f%c"," dTAS:  %7.2f%c","","",1.0/0.3048,units,G->AirspeedRate);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, WHITE );
	  DisplayEngUnit2(buf,"  %7.2f%c","  %7.2f%c","   m/s²","   ft/s²",1.0/0.3048,units,G->RefAirspeedRate);
      TextOut(hDC,mid,line(l),buf,strlen(buf));

      break;

//
// MFD Mode 4 ... Horizontal Status / HAC turn mode
//
    case 4: // Horizontal Status
      SelectDefaultFont(hDC,1);
      HScale(hDC,3,"%6.1f%c°","%6.1f%c°",1.0,G->DelAz*DEG,-20,20,line(23),true, -5, 5);

	  l=1;
	  if (int(G->leg)==3) {
          sprintf(buf,"%s - %s",G->RunwayName[G->runway], ((int(G->leg)==1)? "Descent" : (int(G->leg)==2)? "On HAC" : "On Final" ));
	  } else {
         sprintf(buf,"%s - %s (hac:%ik %c-%c)",G->RunwayName[G->runway], ((int(G->leg)==1)? "Descent" : (int(G->leg)==2)? "On HAC" : "On Final" ),
			 G->HAC_RADIUS/1000, G->hac_sign==1? 'L' : 'R', G->hac_open==1? 'O' : 'C');
	  }
      TextOut(hDC,5,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit(buf,"DelAz:      %9.3f%c°",G->DelAz*DEG);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit2(buf,"Range:    %9.1f   %c", "Range:      %9.3f%c","m","NMi",1.0/1852.0,units,G->range);
      TextOut(hDC,5,line(l),buf,strlen(buf));

  	  switch (G->leg) {
	    case 1:
            l++;
            DisplayEngUnit2(buf,"HAC WP1 Range:%9.1f   %c", "HAC WP1 Range:%9.3f%c","m","NMi",1.0/1852.0,units,G->range_wp1_sc);
            TextOut(hDC,5,line(l),buf,strlen(buf));
            l++;
            DisplayEngUnit(buf,"HAC WP1 Bearing:%9.3f%c°",G->az_tgt_sc*DEG);
	        TextOut(hDC,5,line(l),buf,strlen(buf));
			break;
	    case 2:
            l++;
            DisplayEngUnit2(buf,"HAC WP2 Range:%9.1f   %c", "HAC WP2 Range:%9.3f%c","m","NMi",1.0/1852.0,units,G->arc_range);
            TextOut(hDC,5,line(l),buf,strlen(buf));
            l++;
            DisplayEngUnit(buf,"HAC WP2 Arc:   %6.0f%c°",G->hac_arc*DEG);
	        TextOut(hDC,5,line(l),buf,strlen(buf));
			break;
	    case 3:
            l++;
            DisplayEngUnit2(buf,"PAPI Range:%9.1f   %c", "PAPI Range:%9.3f%c","m","NMi",1.0/1852.0,units,G->range_papi_sc);
            TextOut(hDC,5,line(l),buf,strlen(buf));
            l++;
            DisplayEngUnit2(buf,"PAPI XRange:%9.1f   %c", "PAPI XRange:%9.3f%c","m","NMi",1.0/1852.0,units,G->crossrange);
            TextOut(hDC,5,line(l),buf,strlen(buf));
			break;
	  }

	  
      if (!ManZoomToggle) {
        if(G->range<ZoomMap*1.5) ZoomMap/=1.1;
        if(G->range>ZoomMap*2) ZoomMap*=1.1;
	    if(ZoomMap<MinZoomMap)ZoomMap=MinZoomMap;
	  }
	  turnRadius=G->Groundspeed/G->BearingRate;
      ar_tc_sc.range=turnRadius;
      ar_tc_sc.az=G->Bearing+PI/2;
      nre_tc_sc=az_range_to_nre(ar_tc_sc,G->re);
      xyz_tc=tmul(G->M_sc,nre_tc_sc);
      M_tc=m_nre_to_xyz(xyz_tc);
      nre_sc_tc=mul(M_tc,G->xyz_sc);
      ar_sc_tc=nre_to_az_range(nre_sc_tc,G->re);
      SelectDefaultPen(hDC,0);
      for(int i=10;i<=60;i+=10) {
        AZ_RANGE ar_tp_tc=ar_sc_tc;
        ar_tp_tc.az+=G->BearingRate*i;
        VECTOR3 nre_tp_tc=az_range_to_nre(ar_tp_tc,G->re);
        VECTOR3 xyz_tp=tmul(M_tc,nre_tp_tc);
        VECTOR3 nre_tp_sc=mul(G->M_sc,xyz_tp);
        AZ_RANGE ar_tp_sc=nre_to_az_range(nre_tp_sc,G->re,G->Bearing);
        MoveToEx(hDC,MapX(ar_tp_sc)-4,MapY(ar_tp_sc),NULL);
        LineTo(hDC,MapX(ar_tp_sc)+5,MapY(ar_tp_sc));
        MoveToEx(hDC,MapX(ar_tp_sc),MapY(ar_tp_sc)-4,NULL);
        LineTo(hDC,MapX(ar_tp_sc),MapY(ar_tp_sc)+5);
      }

	  SelectDefaultPen(hDC,2);												// Draw the runway in bright green
      ar_feature_sc=nre_to_az_range(G->nre_far_sc,G->re,G->Bearing);
      MoveToEx(hDC,MapX(ar_feature_sc),MapY(ar_feature_sc),NULL);
      ar_feature_sc=nre_to_az_range(G->nre_near_sc,G->re,G->Bearing);
      LineTo(hDC,MapX(ar_feature_sc),MapY(ar_feature_sc));
      SelectDefaultPen(hDC,3);												// Draw final from runway to WP2 (exit of HAC)
      ar_feature_sc=nre_to_az_range(G->nre_wp2_sc,G->re,G->Bearing);
      LineTo(hDC,MapX(ar_feature_sc),MapY(ar_feature_sc));

	  if (G->hac_open==0) {													// If HAC is open, we fly WP1 to WP2 then down finals, else we do a whole circle. 
        ar_feature_sc=nre_to_az_range(G->nre_hac_sc,G->re,G->Bearing);		// Regular code to draw the HAC circle
        Ellipse(hDC,(int) (MapX(ar_feature_sc)-G->radius_hac*width/ZoomMap),
                  (int) (MapY(ar_feature_sc)-G->radius_hac*height/ZoomMap),
                  (int) (MapX(ar_feature_sc)+G->radius_hac*width/ZoomMap),
                  (int) (MapY(ar_feature_sc)+G->radius_hac*height/ZoomMap));
	  } else {
        ar_feature_wp1=nre_to_az_range(G->nre_wp1_sc,G->re,G->Bearing);		// Pick up the WP1 point
        ar_feature_wp2=nre_to_az_range(G->nre_wp2_sc,G->re,G->Bearing);		// Pick up the WP2 point
	    if (G->hac_sign==1) {
		  SetArcDirection(hDC, AD_COUNTERCLOCKWISE);						// Arc direction anticlockwise per HAC Sign
	    } else {
		  SetArcDirection(hDC, AD_CLOCKWISE);								// Arc direction clockwise per HAC sign
	    }
	    ar_feature_sc=nre_to_az_range(G->nre_hac_sc,G->re,G->Bearing);		// Draw a HAC ARC from WP1 to WP2
        Arc(hDC,(int) (MapX(ar_feature_sc)-G->radius_hac*width/ZoomMap),
                  (int) (MapY(ar_feature_sc)-G->radius_hac*height/ZoomMap),
                  (int) (MapX(ar_feature_sc)+G->radius_hac*width/ZoomMap),
                  (int) (MapY(ar_feature_sc)+G->radius_hac*height/ZoomMap),
				  MapX(ar_feature_wp1),MapY(ar_feature_wp1),
				  MapX(ar_feature_wp2),MapY(ar_feature_wp2));
	  }

      if(G->leg==1) {
        MoveToEx(hDC,MapX(_AR(0,0)),MapY(_AR(0,0)),NULL);					// Draw bottom of screen to WP1 (entry to HAC)
        ar_feature_sc=nre_to_az_range(G->nre_wp1_sc,G->re,G->Bearing);
        LineTo(hDC,MapX(ar_feature_sc),MapY(ar_feature_sc));
      }
      //Glideslope scale
      VScale(hDC,3,"%4.1f°","%4.1f°",1.0,20.0,atan(G->Altitude/G->range_papi_sc)*DEG,10,30,int(8.0*width/10), 18.0, 22.0);
      //Draw track
      if(G->TrackPtr>0) {
        SelectDefaultPen(hDC,1);
        VECTOR3 nre_trackpt_sc=mul(G->M_sc,G->xyz_track[0]);
        ar_feature_sc=nre_to_az_range(nre_trackpt_sc,G->re,G->Bearing);

        MoveToEx(hDC,MapX(ar_feature_sc),MapY(ar_feature_sc),NULL);
        for(int i=1;i<G->TrackPtr;i++) {
          nre_trackpt_sc=mul(G->M_sc,G->xyz_track[i]);
          ar_feature_sc=nre_to_az_range(nre_trackpt_sc,G->re,G->Bearing);
          LineTo(hDC,MapX(ar_feature_sc),MapY(ar_feature_sc));
        }
      }
      break;

//
// MFD Mode 5 ... Internal diagnostics mode ... Enable by putting INTERNAL DIAG on a new line in your GS2.CFG file. 
//
    case 5: // Diagnostics Page
      SetTextColor( hDC, WHITE );
	  l=1;
	  sprintf(buf,"%s %+.3f %+.3f",G->BaseName[G->RunwayBase[G->runway]], G->BaseLL[G->RunwayBase[G->runway]][1],G->BaseLL[G->RunwayBase[G->runway]][0]);
      TextOut(hDC,5,line(l),buf,strlen(buf));
	  l++;
      sprintf(buf,"%s %i %i %i %i",G->RunwayName[G->runway],
		  int(G->RunwayData[G->RunwayBase[G->runway]][2]),
		  int(G->RunwayData[G->RunwayBase[G->runway]][3]),
		  int(G->RunwayData[G->RunwayBase[G->runway]][4]),
		  int(G->RunwayData[G->RunwayBase[G->runway]][5]));
      TextOut(hDC,5,line(l),buf,strlen(buf));

	  l++;
	  DisplayEngUnit(buf,"Energy:%9.3f%cJ",G->TE);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      SetTextColor( hDC, BLUE );
      DisplayEngUnit(buf,"    %9.3f%cJ",G->RefTE);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
      l++;
      SetTextColor( hDC, WHITE );
      DisplayEngUnit(buf,"Mach:  %9.3f%c",G->Mach);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit(buf,"Airspd:%9.3f%cm/s",G->Airspeed);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      DisplayEngUnit(buf,"    %9.3f%ckt",G->Airspeed*3600.0/1852.0);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit(buf,"    %9.3f%cft/s",G->Airspeed/0.3048);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit(buf,"Ispeed:%9.3f%cm/s",G->InertialSpd);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      DisplayEngUnit(buf,"    %9.3f%ckt",G->InertialSpd*3600.0/1852.0);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit(buf,"    %9.3f%cft/s",G->InertialSpd/0.3048);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit(buf,"Range: %9.3f%cm",G->range);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      DisplayEngUnit(buf,"    %9.3f%cNmi",G->range/1852.0);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit(buf,"Alt:   %9.3f%cm",G->Altitude);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      DisplayEngUnit(buf,"    %9.3f%cft",G->Altitude/0.3048);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit(buf,"    %9.3f%cNmi",G->Altitude/1852.0);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit(buf,"Vspd:  %9.3f%cm/s",G->vspd);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      DisplayEngUnit(buf,"    %9.3f%cft/s",G->vspd/0.3048);
      TextOut(hDC,mid,line(l),buf,strlen(buf));
      l++;
      if(G->hasRefSlope) {
        SetTextColor( hDC, BLUE );
        DisplayEngUnit(buf,"RVspd: %9.3f%cm/s",G->RefVspd);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        DisplayEngUnit(buf,"    %9.3f%cft/s",G->RefVspd/0.3048);
        TextOut(hDC,mid,line(l),buf,strlen(buf));
        SetTextColor( hDC, WHITE );
      }
      l++;
      DisplayEngUnit(buf,"HFlux: %9.3f%cW/cm²",G->HeatFlux/10000.0);
      TextOut(hDC,5,line(l),buf,strlen(buf));
      l++;
      DisplayEngUnit(buf,"DelAz: %9.3f%c°",G->DelAz*180.0/PI);
      TextOut(hDC,5,line(l),buf,strlen(buf));
//sprintf(oapiDebugString(),"");
	  break;

//
// MFD Mode 6 ... Deorbit mode
//
    case 6: // Deorbit parameters page
      SetTextColor( hDC, WHITE );
      if(G->Altitude<G->EI_ALT) {
        strcpy(buf,"Entry Interface has occurred");
        TextOut(hDC,5,line(l),buf,strlen(buf));
      } else {
        ELEMENTS E;
        ORBITPARAM OP;
        vessel->GetElements(NULL,E,&OP);
        double PeAlt=OP.PeD-G->re;
        DisplayEngUnit2(buf,"PeAlt: %9.3f%c","PeAlt: %9.3f%c","m","NMi",1.0/1852.0,units,PeAlt);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
        if(PeAlt>G->EI_ALT) {
          strcpy(buf,"No entry predicted");
          TextOut(hDC,5,line(l),buf,strlen(buf));
        } else {
          strcpy(buf,"Entry predicted");
          TextOut(hDC,5,line(l),buf,strlen(buf));
        }
        l++;
        strcpy(buf,"Burn");
        TextOut(hDC,5,line(l),buf,strlen(buf));
        strcpy(buf,"Current");
        TextOut(hDC,mid,line(l),buf,strlen(buf));
        l++;
        double DeorbitHa=OP.ApD-G->re;
        DisplayEngUnit2(buf,"Ha: %9.3f%c","Ha: %9.3f%c","m","NMi",1.0/1852.0,units,DeorbitHa);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        DisplayEngUnit2(buf,"Ha: %9.3f%c","Ha: %9.3f%c","m","NMi",1.0/1852.0,units,OP.ApD-G->re);
        TextOut(hDC,mid,line(l),buf,strlen(buf));
        l++;
        double DeorbitRa=DeorbitHa+G->re;
        double REI=(G->re+G->EI_ALT);
        DisplayEngUnit2(buf,"Hp: %9.3f%c","Hp: %9.3f%c","m","NMi",1.0/1852.0,units,G->DeorbitHp);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        DisplayEngUnit2(buf,"Hp: %9.3f%c","Hp: %9.3f%c","m","NMi",1.0/1852.0,units,OP.PeD-G->re);
        TextOut(hDC,mid,line(l),buf,strlen(buf));
        l++;
        double DeorbitRp=G->DeorbitHp+G->re;
        VECTOR3 VVi;
        vessel->GetRelativeVel(G->Planet,VVi);
        double Vi=length(VVi);
        double a=(DeorbitRa+DeorbitRp)/2;
        double e=(DeorbitRa-DeorbitRp)/(DeorbitRa+DeorbitRp);
        double p=a*(1-e*e);
        double ViA=sqrt(2*G->GtimesM1/DeorbitRa-G->GtimesM1/a);
        double ViDapat=sqrt(2*G->GtimesM1/(G->re+vessel->GetAltitude())-G->GtimesM1/a);
        double ViP=sqrt(2*G->GtimesM1/DeorbitRp-G->GtimesM1/a);
        double ViEI=sqrt(2*G->GtimesM1/REI-G->GtimesM1/a);
        double TAEI=2*PI-acos(p/(REI*e)-1/e);
        double TA;
        if(E.e<0.001) TA=183*RAD; else TA=OP.TrA;
        double EEI=2*PI+2*atan(sqrt((1-e)/(1+e))*tan(TAEI/2));
        double sFPAEI=e*sin(EEI)/sqrt(1-pow(e*cos(EEI),2));
        double vspdEI=ViEI*sFPAEI;
        DisplayEngUnit2(buf,"Hd: %9.3f%c","Hd: %9.3f%c","m/s","ft/s",1.0/0.308,units,vspdEI);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
        double DeorbitDeltaV=Vi-ViDapat;
        DisplayEngUnit2(buf,"DV: %9.3f%c","DV: %9.3f%c","m/s","ft/s",1.0/0.308,units,DeorbitDeltaV);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
        double F,Isp,mdot;
        getMainThrustParm(vessel, &F, &Isp);
        mdot=F/Isp;
        double DeorbitT=(DeorbitDeltaV*vessel->GetMass()/(2*F))*(1+exp(-1*DeorbitDeltaV/Isp));
        DisplayEngUnit(buf,"T:  %9.3f%cs",DeorbitT);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
        double Vavg=(Vi+ViDapat)/2;
        DisplayEngUnit2(buf,"Vi:     %9.3f%c","Vi:     %9.3f%c","m/s","ft/s",1.0/1852.0,units,Vi);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
        DisplayEngUnit2(buf,"Vdpt:   %9.3f%c","Vdpt:   %9.3f%c","m/s","ft/s",1.0/1852.0,units,ViDapat);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
        DisplayEngUnit2(buf,"Vavg:   %9.3f%c","Vavg:   %9.3f%c","m/s","ft/s",1.0/1852.0,units,Vavg);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
        double DeorbitDist=Vavg*DeorbitT;
        double DeorbitArc=DeorbitDist/G->re;
        double CoastStartTA=TA+DeorbitArc;
        double CoastEndTA=TAEI;
        double CoastArc=CoastEndTA-CoastStartTA;
        double RangeToEI=CoastArc*(G->re)+DeorbitDist;
        DisplayEngUnit2(buf,"Dburn: %9.3f%c","Dburn: %9.3f%c","m","NMi",1.0/1852.0,units,DeorbitDist);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
        DisplayEngUnit2(buf,"DEI:   %9.3f%c","DEI:   %9.3f%c","m","NMi",1.0/1852.0,units,RangeToEI);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
        double DMG=RangeToEI*cos(G->DelAz);
        DisplayEngUnit2(buf,"GEI:   %9.3f%c","GEI:   %9.3f%c","m","NMi",1.0/1852.0,units,DMG);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
        double RangeAtEI=G->range-DMG;
        DisplayEngUnit2(buf,"EIrng: %9.3f%c","EIrng: %9.3f%c","m","NMi",1.0/1852.0,units,RangeAtEI);
        TextOut(hDC,5,line(l),buf,strlen(buf));
        l++;
      }
      break;

//
// MFD Mode 100 ... Status after user slope save
//
    case 100: // Deorbit parameters page
      SetTextColor( hDC, WHITE );
      strcpy(buf,"GS2glideslope_UserSave.cfg created.");
      TextOut(hDC,5,line(5),buf,strlen(buf));
      strcpy(buf,"Hit MOD to return to previous mode.");
      TextOut(hDC,5,line(8),buf,strlen(buf));
	  break;
  }
}

void Glideslope::VTape(HDC hDC,char *title,char *metUnit,char* usUnit,int radius, int minor,double usFactor,double ref,double act,int x1, int x2) {
  char actbuf[256];
  char refbuf[256];
  char buf[256];
  int y;
  int title_pos=line(4);
  int top=line(5);
  int bot=line(21);
  char sym = '?';
  int pwr = -99;


  // sprintf(oapiDebugString(),"cw %i ch %i",cw, ch);


  HGDIOBJ oldBrush=SelectObject(hDC,blueBrush);
  HGDIOBJ oldPen=SelectObject(hDC,whitePen);
  Rectangle(hDC,x1,top,x2,bot);
  double ybot,ytop;
  if(!units) {
    ybot=act-radius;
    ytop=act+radius;
	if (act>ref) {
      DisplayEngUnit3(actbuf,"%4.1f%c",act,&sym,&pwr,0,15);
	  DisplayEngUnit4(refbuf,"%4.1f%c",ref,sym,pwr);
	} else {
      DisplayEngUnit3(refbuf,"%4.1f%c",ref,&sym,&pwr,0,15);
	  DisplayEngUnit4(actbuf,"%4.1f%c",act,sym,pwr);
	}
	sprintf(buf, "%c%s", sym, metUnit);
    TextOut(hDC,x1,bot,buf,strlen(buf));
    y=(int) linterp(ybot,bot,ytop,top,ref, true);
  } else {
    ybot=act*usFactor-radius;
    ytop=act*usFactor+radius;
	if (act>ref) {
      DisplayEngUnit3(actbuf,"%4.1f%c",act*usFactor,&sym,&pwr,0,15);
      DisplayEngUnit4(refbuf,"%4.1f%c",ref*usFactor,sym,pwr);
	} else {
      DisplayEngUnit3(refbuf,"%4.1f%c",ref*usFactor,&sym,&pwr,0,15);
      DisplayEngUnit4(actbuf,"%4.1f%c",act*usFactor,sym,pwr);
	}
	sprintf(buf, "%c%s", sym, usUnit);
    TextOut(hDC,x1,bot,buf,strlen(buf));
    y=(int) linterp(ybot,bot,ytop,top,ref*usFactor, true);
  }
  if(y<(top+5))y=top+5;
  if(y>(bot-5))y=bot+8-ch;
  double firstlabel=floor(ybot/minor)*minor;
  double lastlabel=(ceil(ytop/minor)+1)*minor;
  MoveToEx(hDC,x1,y,NULL);
  LineTo(hDC,x2,y);
  TextOut(hDC,x1-2,title_pos,title,strlen(title));
  HRGN hrgn = CreateRectRgn(x1+1,top+1,x2-1,bot-1);
  SelectClipRgn(hDC, hrgn);
  for(double i=firstlabel;i<lastlabel;i+=minor) {
    if(i>=ref) SetTextColor( hDC, WHITE ); else SetTextColor(hDC,RED);
    DisplayEngUnit4(buf,"%4.1f%c",i,sym,pwr);
    int yy=(int) linterp(ybot,bot,ytop,top,i, true);
    TextOut(hDC,x1,yy-6,buf,strlen(buf));
  }
  SelectClipRgn(hDC,NULL);
  SelectObject(hDC,blueBrush);
  Rectangle(hDC,x1,y-6,x2,y+ch-3);   // ch is the default char height for the panel (it gets resized according to ship and F8 mode)
  SelectClipRgn(hDC, hrgn);
  SetTextColor(hDC,WHITE);
  TextOut(hDC,x1,y-6,refbuf,strlen(refbuf));
  SelectClipRgn(hDC,NULL);
  Rectangle(hDC,x1,(bot+top)/2-6,x2,(bot+top)/2+ch-3);
  SelectClipRgn(hDC, hrgn);

  if ((!units) && (abs(act-ref)*usFactor< minor)) {
    SetTextColor( hDC, GREEN );
  } else if ((units) && (abs(act-ref)*usFactor < minor)) {
    SetTextColor( hDC, GREEN );
  } else if(act<ref) {
    SetTextColor( hDC, RED );
  } else {
	SetTextColor(hDC, WHITE);
  }
  
  TextOut(hDC,x1,(bot+top)/2-6,actbuf,strlen(actbuf));
  SelectClipRgn(hDC,NULL);
  DeleteObject(hrgn);
  SetTextColor(hDC,WHITE);
  SelectObject(hDC,oldBrush);
  SelectObject(hDC,oldPen);
}

void Glideslope::VScale(HDC hDC,int pen,char* metFmt,char* usFmt,double usFactor, double ref, double act, double min, double max, int x, double lowarn, double hiwarn) {
  char buf[256];
  int y;
  SelectDefaultPen(hDC,pen);
  int top=line(5);
  int bot=line(21);
  MoveToEx(hDC,x-5,top,NULL);
  LineTo(hDC,x+5,top);
  MoveToEx(hDC,x-5,bot,NULL);
  LineTo(hDC,x+5,bot);
  MoveToEx(hDC,x,top,NULL);
  LineTo(hDC,x,bot);
  y=(int) linterp(min,bot,max,top,ref, true);
  MoveToEx(hDC,x-5,y-5,NULL);
  LineTo(hDC,x,y);
  LineTo(hDC,x-5,y+5);
  if(!units) {
    DisplayEngUnit4(buf,metFmt,ref, ' ', 0);
  } else {
    DisplayEngUnit4(buf,usFmt,ref*usFactor, ' ', 0);
  }
  UINT OldAlign=SetTextAlign(hDC,TA_TOP+TA_RIGHT);
  SetTextColor( hDC, BLUE );
  TextOut(hDC,x-5,y-5,buf,strlen(buf));
  y=(int) linterp(min,bot,max,top,act, true);

//sprintf(oapiDebugString(),"Vscale ref: x1(%4.1f) x2(%4.1f), y1(%4.1f) y2(%4.1f), low(%4.1f) hi(%4.1f) x(%4.1f) ==>y(%4.1f)", min, max, bot, top, lowarn, hiwarn, act, y);  

  MoveToEx(hDC,x+5,y-5,NULL);
  LineTo(hDC,x,y);
  LineTo(hDC,x+5,y+5);
  if(!units) {
    DisplayEngUnit4(buf,metFmt,act, ' ', 0);
  } else {
    DisplayEngUnit4(buf,usFmt,act*usFactor, ' ', 0);
  }
  SetTextAlign(hDC,TA_TOP+TA_LEFT);
  SetTextColor( hDC, ((act<=lowarn) ? RED : (act>=hiwarn) ? WHITE : GREEN ));
  TextOut(hDC,x+5,y-5,buf,strlen(buf));
  SetTextColor( hDC, WHITE );
  SetTextAlign(hDC,OldAlign);
}

void Glideslope::HScale(HDC hDC,int pen,char* metFmt,char* usFmt,double usFactor, double act, double min, double max, int y, bool top, double lowarn, double hiwarn) {
  char buf[256];
  int x;
  SelectDefaultPen(hDC,pen);
  MoveToEx(hDC,5,y-5,NULL);
  LineTo(hDC,5,y+5);
  MoveToEx(hDC,width-5,y-5,NULL);
  LineTo(hDC,width-5,y+5);
  MoveToEx(hDC,5,y,NULL);
  LineTo(hDC,width-5,y);
  x=(int) linterp(min,5,max,width-5,0, true);
  MoveToEx(hDC,x,y-5,NULL);
  LineTo(hDC,x,y+5);
  x=(int) linterp(min,5,max,width-5,act, true);
  if(x>width-5) {
    SetTextColor(hDC,RED);
    x=width-5;
  }
  if(x<5){
    x=5;
    SetTextColor(hDC,RED);
  }
  MoveToEx(hDC,x-5,y+(top?1:-1)*5,NULL);
  LineTo(hDC,x,y);
  LineTo(hDC,x+5,y+(top?1:-1)*5);
  if(!units) {
    DisplayEngUnit4(buf,metFmt,act, ' ', 0);
  } else {
    DisplayEngUnit4(buf,usFmt,act*usFactor, ' ', 0);
  }
  UINT OldAlign=SetTextAlign(hDC,(top?TA_TOP:TA_BOTTOM)+TA_LEFT);
  SetTextColor(hDC,((act <= lowarn)? RED : (act>= hiwarn) ? WHITE : GREEN));
  TextOut(hDC,x-5,line(24),buf,strlen(buf));
  SetTextAlign(hDC,OldAlign);
  SetTextColor(hDC,WHITE);
}

void Glideslope::getMainThrustParm(VESSEL* vessel, double *F, double *isp) {
  *F = 0;
  *isp = 0;
  int nthr=vessel->GetGroupThrusterCount(THGROUP_MAIN);
  for(int i=0; i<nthr; i++) {
    THRUSTER_HANDLE th=vessel->GetGroupThruster(THGROUP_MAIN,i);
    *F += vessel->GetThrusterMax0(th);
    *isp += vessel->GetThrusterIsp(th) * vessel->GetThrusterMax0(th);
  }
  *isp /= *F;
}



//
// ==============================================================
// Orbiter DLL API interface
// ==============================================================
//

DLLCLBK void opcDLLInit (HINSTANCE hDLL) {
  static char *name = "Glideslope 2";   // MFD mode name .. enhancing the original awesome Glideslope into Glideslope 2
  MFDMODESPEC spec;
  spec.name = name;
  spec.key = OAPI_KEY_W;                // MFD mode selection key
  spec.msgproc = Glideslope::MsgProc;  // MFD mode callback function

  // Register the new MFD mode with Orbiter
  g_MFDmode = oapiRegisterMFDMode (spec);
  nGutsUsed=0;
}

DLLCLBK void opcDLLExit (HINSTANCE hDLL) {
  // Unregister the custom MFD mode when the module is unloaded
  oapiUnregisterMFDMode (g_MFDmode);
}

DLLCLBK void opcPreStep(double SimT,double SimDT,double mjd) {
  for(int i=0;i<nGutsUsed;i++) {
    if(gg[i].isInit)gg[i].MinorCycle(SimT);
  }
}

// MFD message parser
int Glideslope::MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam) {
  switch (msg) {

  case OAPI_MSG_MFD_OPENED:
    // Our new MFD mode has been selected, so we create the MFD and
    // return a pointer to it.
    return (int)(new Glideslope (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam, mfd));
  }
  return 0;
}

bool Glideslope::ConsumeKeyBuffered (DWORD key) {
  switch (key) {
    case OAPI_KEY_A:
      G->runway--;
      if(G->runway<0) G->runway=G->RunwayCount-1;
      G->calcRunway();
      return true;
    case OAPI_KEY_S:
      G->runway++;
      if(G->runway>=G->RunwayCount) G->runway=0;
      G->calcRunway();
      return true;
    case OAPI_KEY_R:		// Reset HAC, Track, Zoom 
	  ManZoomToggle = false;
	  G->HAC_RADIUS = 15000;
	  G->FINAL_DIST=20000;
      G->hac_sign=1;
      G->calcRunway();
      return true;
    case OAPI_KEY_M:
	  if (mode>=100) {		// 100 range .. message screens ... hit MOD to jump mode back to previous
	    mode = jumpmode;
	  } else {
        mode++;
	  }
      if(mode>G->MaxMOD) mode=1;
	  G->gutsMode[gutsMFDref] = mode;
      return true;
    case OAPI_KEY_U:
      units=!units;
      return true;
    case OAPI_KEY_H:
	  if (G->hac_open == 1) {
		G->hac_sign=-G->hac_sign;
		G->hac_open = 0;
	  } else {
		G->hac_open = 1;
	  }
      G->calcRunway();
      return true;
    case OAPI_KEY_B:
      G->steerActive=!G->steerActive;
      //Shut them off here, but just once so that other autopilots can still play
      vessel->SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT,0);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT,0);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,0);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0);
      return true;
    case OAPI_KEY_V:
	  jumpmode = mode;
	  mode = 100;
  	  G->gutsMode[gutsMFDref] = mode;
      G->Save(units);
      return true;
    case OAPI_KEY_EQUALS:		// HAC++
      if (G->HAC_RADIUS<45000) {		// Max 3x default, else reset to default. 
		G->HAC_RADIUS += 500;
  	    G->FINAL_DIST += 500;
	    G->calcRunway();
	  } else {
	    G->HAC_RADIUS = 15000;
	    G->FINAL_DIST=20000;
	  }
      return true;
    case OAPI_KEY_MINUS:		// HAC--
      if (G->HAC_RADIUS>500) G->HAC_RADIUS -= 500;	// Smallest is a 500m HAC !! 
	  if (G->FINAL_DIST>8000) G->FINAL_DIST -= 500;	// Set a minimum final run of 8km
	  G->calcRunway();
      return true;
    case OAPI_KEY_SEMICOLON:	// HZoom--
	  ManZoomToggle = true;
      ZoomMap/=1.1;
      if(ZoomMap<MinZoomMap)ZoomMap=MinZoomMap;
      return true;
    case OAPI_KEY_APOSTROPHE:	// HZoom++
	  ManZoomToggle = true;
      ZoomMap*=1.1;
      return true;
  }
  return false;
}

char *Glideslope::ButtonLabel (int bt) {
  char *label[12] = {"PR","NR","MOD","UNT","AUT","SAV","HAC","RST","HR+","HR-","ZM+","ZM-"};
  return (bt < 12 ? label[bt] : 0);
}

int Glideslope::ButtonMenu (const MFDBUTTONMENU **menu) const
{
  static const MFDBUTTONMENU mnu[12] = {
    {"Previous Rwy", 0, 'A'},
    {"Next Rwy", 0, 'S'},
    {"Switch Mode", 0, 'M'},
    {"Switch Units", 0, 'U'},
    {"AutoPilot Toggle", 0, 'B'},
    {"Save User Glideslope", 0, 'V'},
    {"Switch HAC", 0, 'H'},
    {"Reset HAC & Zoom", 0, 'R'},
	{"HAC Radius++", 0, '='},
    {"HAC Radius--", 0, '-'},
    {"Manual Zoom++", 0, '\''},
    {"Manual Zoom--", 0, ';'}
  };
  if (menu) *menu = mnu;
  return 12;
}

bool Glideslope::ConsumeButton (int bt, int event) {
  if (!(event & PANEL_MOUSE_LBDOWN)) return false;
  static const DWORD btkey[12] = { OAPI_KEY_A,OAPI_KEY_S,OAPI_KEY_M,OAPI_KEY_U,OAPI_KEY_B,OAPI_KEY_V,OAPI_KEY_H,OAPI_KEY_R,OAPI_KEY_EQUALS,OAPI_KEY_MINUS,OAPI_KEY_SEMICOLON,OAPI_KEY_APOSTROPHE};
  if (bt < 12) return ConsumeKeyBuffered (btkey[bt]);
  else return false;
}


void Glideslope::LoadConfig () {
  FILE* GS2cfg;
//  int scancount;
  char buf[256];
  char bufcpy[256];
  buf[255] = '\0';
  char *tok=buf;
  char *strtok=buf;
  double tokf[6];
  int j;
  int ofs;
  int modeLtok = 0;
  int modeRtok = 0;
  int modeXtok = 0;
  bool unitstok = false;

  char *bp;
  bool prefsFound = false, runwayFound = false, baseFound = false; 

  GS2cfg = fopen(".\\Config\\MFD\\GS2\\GS2.cfg","r");
  if (GS2cfg == NULL) {
// Defaults to init code
	  LoadDefaultConfig();
	  return;
  }

  G->MaxMOD=4;
  G->HAC_RADIUS = 15000;
  G->FINAL_DIST = 20000;
  G->BaseCount = 0;
  G->RunwayCount = 0;

  FILE* GS2CfgPL=fopen(".\\Config\\MFD\\GS2\\Logs\\GS2cfgParselog.txt","w");
  fprintf(GS2CfgPL,"Successfully found and opened GS2.cfg\n");

  while (fgets(buf,255,GS2cfg)!=NULL) {
	bp=buf;
	sprintf(bufcpy,"%s",buf);
	while (bp[0]==' ' || bp[0]=='\t') bp++;
	if ((bp[0]=='\0')||(bp[0]=='\n')) {
      fprintf(GS2CfgPL,"NUL> %s", buf);
	  continue;
	}
	if (bp[0]==';') {
      fprintf(GS2CfgPL,"REM> %s", buf);
	  continue;
	}

	if (!ParseString(&bp,&tok)) goto badparse;

	if (_stricmp(tok,"BASE")==0)
	{
// Parsing BASE "base name" lon lat
// e.g.    BASE "Cape Canaveral" -80.675 +28.5208

	  if (!ParseQuotedString(&bp,&strtok)) goto badparse;
	  if (strlen(strtok)>32) goto badparse;
	  for (j=0; j<2; j++) {
	    if (!ParseDouble(&bp,&(tokf[j]))) goto badparse;
	  }
	  if (G->BaseCount >= G->BaseLimit) goto badparse;
	  sprintf(G->BaseName[G->BaseCount],strtok);
	  G->BaseLL[G->BaseCount][0] = tokf[1];
	  G->BaseLL[G->BaseCount][1] = tokf[0];
	  fprintf(GS2CfgPL,"BASE LOADED: %s Lon:%.3f Lat:%.3f\n", G->BaseName[G->BaseCount],G->BaseLL[G->BaseCount][1],G->BaseLL[G->BaseCount][0] );
	  baseFound = true;
	  G->BaseCount++;
	  continue;
	}

	if (_stricmp(tok,"RUNWAY")==0)
	{
// Parsing RUNWAY "{basename}" "{rwyname}" {end1-1} {end1-3} {end2-1} {end2-3} {papi} {vasi}
// e.g.    RUNWAY "Cape Canaveral" "KSC33" -8220 -600 -12670 -3155 -2000 671

	  if (!ParseQuotedString(&bp,&strtok)) goto badparse;
	  
	  for (ofs=0; (ofs<G->BaseCount) && (_stricmp(strtok,(G->BaseName[ofs]))!=0) ; ofs++) {}
	  if (ofs==G->BaseCount) goto badparse;

	  if (!ParseQuotedString(&bp,&strtok))  goto badparse;
	  if (strlen(strtok)>12) goto badparse;

	  for (j=0; j<6; j++) {
	    if (!ParseDouble(&bp,&(tokf[j]))) goto badparse;
	  }
	  if (G->RunwayCount >= G->RunwayLimit) goto badparse;

	  sprintf(G->RunwayName[G->RunwayCount],strtok);
	  for (j=0;j<2;j++) G->RunwayData[G->RunwayCount][j] = G->BaseLL[ofs][j];
	  for (j=0;j<6;j++) G->RunwayData[G->RunwayCount][j+2] = tokf[j];
	  G->RunwayBase[G->RunwayCount] = ofs;

	  fprintf(GS2CfgPL,"RWY LOADED: %s\'s %s END1(%.0f, %.0f) END2(%.0f, %.0f) PAPI=%.0f VASI=%.0f\n", G->BaseName[ofs], G->RunwayName[G->RunwayCount], 
		  G->RunwayData[G->RunwayCount][2], G->RunwayData[G->RunwayCount][3], G->RunwayData[G->RunwayCount][4],
		  G->RunwayData[G->RunwayCount][5], G->RunwayData[G->RunwayCount][6], G->RunwayData[G->RunwayCount][7]);
	  runwayFound = true;
	  G->RunwayCount++;
	  continue;
	}

	if (_stricmp(tok,"PREFS")==0)
	{

// Parsing PREFS {mode-L} {mode-R} {mode-X} {units}        {rwy}   {glideslope}
// e.g.    PREFS 2        3        4        {US | METRIC}  "KSC33" XR

	  if (prefsFound) goto badparse;

	  if (!ParseInt(&bp,&modeLtok)) goto badparse;
	  if (!ParseInt(&bp,&modeRtok)) goto badparse;
	  if (!ParseInt(&bp,&modeXtok)) goto badparse;
	  if (!ParseString(&bp,&tok)) goto badparse;
	  if (_stricmp(tok,"US")==0) {
		  unitstok = true;
	  } else if (_stricmp(tok,"METRIC")==0) {
		  unitstok = false;
	  } else goto badparse;
	  if (!ParseQuotedString(&bp,&strtok)) goto badparse;
	  for (ofs=0; (ofs<G->RunwayCount) && (_stricmp(strtok,(G->RunwayName[ofs]))!=0) ; ofs++) {}
	  if (ofs==G->RunwayCount) goto badparse;
	  if (!ParseString(&bp,&tok)) goto badparse;

	  LoadPreferredGlideslopeConfig(tok);

	  G->ParamsModeL = modeLtok;
	  if (G->ParamsModeL<1) G->ParamsModeL=1;
	  if (G->ParamsModeL>G->MaxMOD) G->ParamsModeL=G->MaxMOD;
	  G->ParamsModeR = modeRtok;
	  if (G->ParamsModeR<1) G->ParamsModeR=1;
	  if (G->ParamsModeR>G->MaxMOD) G->ParamsModeR=G->MaxMOD;
	  G->ParamsModeX = modeXtok;
	  if (G->ParamsModeX<1) G->ParamsModeX=1;
	  if (G->ParamsModeX>G->MaxMOD) G->ParamsModeX=G->MaxMOD;

	  G->ParamsUnits = unitstok;
	  G->ParamsRwy = ofs;
	  
	  fprintf(GS2CfgPL,"PREFS LOADED: Mode=%i/%i/%i Units=%s Runway=%s Glideslope=%s\n", G->ParamsModeL,G->ParamsModeR,G->ParamsModeX, (G->ParamsUnits?"US":"METRIC"), G->RunwayName[G->ParamsRwy], tok );
	  G->BaseCount++;
	  prefsFound = true;
	  continue;
		
   }
   if (_stricmp(tok,"INTERNAL")==0)
   {
// Parsing INTERNAL DEBUG FLAG

	  if (!ParseString(&bp,&strtok)) goto badparse;
  	  if (_stricmp(strtok,"DIAG")!=0) goto badparse;
	  G->MaxMOD =6;
	  continue;
	}


   badparse:
   fprintf(GS2CfgPL,"Bad parse, ignored line>> %s", bufcpy);
   fprintf(GS2CfgPL,"(Check for typos, base not previously defined, base>32 chars, runway >12 chars, >64 bases, > 128 rwys, >1 prefs line, etc.)\n\n");
  }

  if (!feof(GS2cfg)) {
    fprintf(GS2CfgPL,"Bad read on GS2.cfg\n");
    LoadDefaultConfig();
  } else {
    if ((!prefsFound)||(!baseFound)||(!runwayFound)) {
      if (!prefsFound) fprintf(GS2CfgPL,"No PREFS found ... defaulting to Default Config");
      if (!baseFound) fprintf(GS2CfgPL,"No BASE found ... defaulting to Default Config");
      if (!runwayFound) fprintf(GS2CfgPL,"No RUNWAY found ... defaulting to Default Config");
      LoadDefaultConfig();
    } else {
      fprintf(GS2CfgPL,"Successfully parsed GS2.cfg\n");
	}
  }
  fclose(GS2CfgPL);
  fclose(GS2cfg);

  return;
}



void Glideslope::LoadDefaultConfig () {

  double DefaultRunwayData[2][8]= {
          //Base lat, deg   Base lon, deg   RW near south, m   RW near east, m, RW far south, m, RW far east, m, Disp VASI, m,  Disp PAPI, m
	  /*KSC33*/{      28.5208,       -80.675,             -8220,              -600,          -12670,          -3155,          671,         -2000},
	  /*KSC15*/{      28.5208,       -80.675,             -12670,          -3155,            -8220,              -600,        671,         -2000}
  
  };
  
  strcpy(G->BaseName[0],"Cape Canaveral (Default)");
  strcpy(G->RunwayName[0], "KSC33 (def)");
  strcpy(G->RunwayName[1], "KSC15 (def)");

  for (int i=0; i<8; i++) {
	  G->RunwayData[0][i] = DefaultRunwayData[0][i];
	  G->RunwayData[1][i] = DefaultRunwayData[1][i];
  }
  G->RunwayBase[0] = 0;
  G->RunwayBase[1] = 0;
  G->RunwayCount = 2;
  G->BaseCount = 1;
  G->ParamsModeL = 2;
  G->ParamsModeR = 3;
  G->ParamsModeX = 4;
  units = FALSE;

/*
sprintf(oapiDebugString(),"Default rwy init Base(%s) Rwy(%s) LatLon(%4.1f %4.1f) Near(%4.1f %4.1f) Far(%4.1f %4.1f) TDP PAPI(%4.1f %4.1f)",
BaseName[RunwayBase[G->runway]], RunwayName[G->runway],
RunwayData[G->runway][0], RunwayData[G->runway][1], 
RunwayData[G->runway][2], RunwayData[G->runway][3], 
RunwayData[G->runway][4], RunwayData[G->runway][5], 
RunwayData[G->runway][6], RunwayData[G->runway][7]); 
*/

  return;
}



void Glideslope::LoadPreferredGlideslopeConfig (char *PrefFile) {
  FILE* GS2gscfg;
  char buf[256];
  char bufcpy[256];
  buf[255] = '\0';
  char *tok=buf;
  char *strtok=buf;
  int i, j;
  int modetok = 0;
  bool unitstok = false;

  double gsparams[G->GlideslopePointsLimit][6];
  int gsrow=0;

  char *bp;
  bool unitsFound = false, gsFound = false, inGSblock = false;
  bool unitMode = false;

  sprintf(buf,".\\Config\\MFD\\GS2\\GS2glideslope_%s.cfg",PrefFile);

  GS2gscfg = fopen(buf,"r");
  if (GS2gscfg == NULL) {
// Defaults to init code
	  LoadDefaultGlideslopeConfig();
	  return;
  }
  sprintf(buf,".\\Config\\MFD\\GS2\\Logs\\GS2glideslope_%s_ParseLog.txt",PrefFile);

  FILE* GS2CfgPL=fopen(buf,"w");
  fprintf(GS2CfgPL,"Successfully found and opened %s\n",buf);

  while (fgets(buf,255,GS2gscfg)!=NULL) {
	bp=buf;
	sprintf(bufcpy,"%s",buf);
	while (bp[0]==' ' || bp[0]=='\t') bp++;
	if ((bp[0]=='\0')||(bp[0]=='\n')) {
      fprintf(GS2CfgPL,"NUL> %s", buf);
	  continue;
	}
	if (bp[0]==';') {
      fprintf(GS2CfgPL,"REM> %s", buf);
	  continue;
	}

	if (!ParseString(&bp,&tok)) goto badparse;

	if (_stricmp(tok,"UNITS")==0)
	{
// Parsing UNITS METRIC or UNITS US

	  if (!ParseString(&bp,&strtok)) goto badparse;
  	  if (_stricmp(strtok,"METRIC")==0) {
		  unitsFound = true;
		  unitMode = false;
          fprintf(GS2CfgPL,"UNITS are Metric\n");
	      continue;
	  } else if (_stricmp(strtok,"US")==0) {
		  unitsFound = true;
		  unitMode = true;
          fprintf(GS2CfgPL,"UNITS are US\n");
		  continue;
	  } else goto badparse;
	}

	if (_stricmp(tok,"BEGIN")==0)
	{
// Parsing BEGIN GLIDESLOPE

	  if (!ParseString(&bp,&strtok)) goto badparse;
  	  if (_stricmp(strtok,"GLIDESLOPE")!=0) goto badparse;
	  if (!unitsFound) goto badparse;
	  if (inGSblock) goto badparse;
	  inGSblock = true;
	  continue;
	}

	if (_stricmp(tok,"END")==0)
	{
// Parsing END GLIDESLOPE

	  if (!ParseString(&bp,&strtok)) goto badparse;
  	  if (_stricmp(strtok,"GLIDESLOPE")!=0) goto badparse;
	  if (!inGSblock) goto badparse;
	  inGSblock = false;
	  continue;
	}

// To get here .... we should be in the GS details 
    if (!inGSblock) goto badparse;
    if (gsrow>G->GlideslopePointsLimit) goto badparse;

	gsparams[gsrow][0] = atoi(tok);
	for (j=1;j<5;j++) {
      if (!ParseDouble(&bp,&(gsparams[gsrow][j]))) goto badparse;
	}
	gsparams[gsrow][5] = 0;
	gsrow++;
    fprintf(GS2CfgPL,"Loaded glideslope point> %s", bufcpy);
    continue;

badparse:
   fprintf(GS2CfgPL,"Bad parse, ignored line>> %s", bufcpy);
   fprintf(GS2CfgPL,"(Check for typos, no UNITS line, badly formed GLIDESLOPE data etc.)\n\n");
  }

  if (!feof(GS2gscfg)) {
    fprintf(GS2CfgPL,"Bad read on GS2 glideslope points file\n");
    LoadDefaultGlideslopeConfig();
  } else {
    if (gsrow<2) {
      fprintf(GS2CfgPL,"Need glideslope points in a glideslope points file ... defaulting to Default Config");
      LoadDefaultGlideslopeConfig();
    } else {
      fprintf(GS2CfgPL,"Successfully parsed GS2 glideslope points file\n");
	}
  }

// Transfer points into master structure
  const int pmap[6] = {2, 3, 0, 4, 1, 5};		// Order from points file different to order in the internal table. Fix it up
  const char colname[6][4] = {"RNG","ALT","TAS","VSP","AOA","ZRO"};
  const double cnvM[6] = {1000, 1000, 1, 1, RAD, 0};	// Conversion for metric points entry Rng, Alt, TAS, Vspd, AOA, 0
  const double cnvU[6] = {1852, 304.8, 0.3048, 0.3048, RAD, 0};	// Conversion for US points entry Rng, Alt, TAS, Vspd, AOA, 0
  double cnv[6];

  for (j=0;j<6;j++) {cnv[j] = (unitMode? cnvU[j] : cnvM[j]);};

  G->RefSlopePts = gsrow;
  for (i=0;i<gsrow;i++) {
      fprintf(GS2CfgPL,"GS2 point loaded: ");
	  for (j=0; j<6; j++) {
        G->RefSlope[i][pmap[j]] = gsparams[i][j]*cnv[j]; 
        fprintf(GS2CfgPL,"%s %.3f (%.3f * %.3f) ",colname[j], G->RefSlope[i][pmap[j]], gsparams[i][j], cnv[j] );
      }
      fprintf(GS2CfgPL,"\n");
  }
  fprintf(GS2CfgPL,"Completed\n");

  fclose(GS2CfgPL);
  fclose(GS2gscfg);

  return;
}

void Glideslope::LoadDefaultGlideslopeConfig () {

    int i, j;
	const double DefSlope[44][6]={


  // New Ref Slope in Meters
  //           TAS m/s         AoA deg        Range, m              Alt, m      dAlt, m/s         Segment length
  //-----------------------------------------------------------
 
  {                   7501,    41.0*RAD,        8145000,             120000,           -95,         0},
  {                   7504,    41.0*RAD,        8000000,             118000,           -95,         0},
  {                   7520,    41.5*RAD,        7000000,             105000,           -94,         0}, 
  {                   7533,    41.5*RAD,        6000000,              93000,           -89,         0},
  {                   7534,    42.5*RAD,        5500000,              87000,           -85,         0},
  {                   7524,    42.5*RAD,        5000000,              81500,           -79,         0},
  {                   7491,    42.0*RAD,        4500000,              76500,           -70,         0},
  {                   7416,    42.0*RAD,        4000000,              72000,           -60,         0},
  {                   7285,    42.0*RAD,        3500000,              68500,           -55,         0},
  {                   7078,    43.0*RAD,        3000000,              65500,           -55,         0}, 
  {                   6774,    43.0*RAD,        2500000,              62000,           -55,         0},
  {                   6245,    40.5*RAD,        2000000,              56000,           -55,         0},
  {                   5352,    40.5*RAD,        1500000,              50500,           -55,         0},
  {                   4176,    36.5*RAD,        1000000,              46500,           -50,         0},
  {                   3652,    36.5*RAD,         800000,              44600,           -50,         0},
  {                   2987,    35.5*RAD,         600000,              41000,           -50,         0},
  {                   2567,    34.0*RAD,         500000,              39000,           -50,         0},
  {                   2294,    33.0*RAD,         450000,              38500,           -50,         0},
  {                   2109,    33.0*RAD,         400000,              37600,           -60,         0}, 
  {                   1908,    33.0*RAD,         350000,              35700,           -90,         0},
  {                   1645,    33.0*RAD,         300000,              33100,           -90,         0},
  {                   1491,    33.0*RAD,         275000,              31800,           -80,         0},
  {                   1320,    33.0*RAD,         250000,              30400,           -70,         0},
  {                   1135,    33.0*RAD,         225000,              28900,           -80,         0},
  {                    919,    33.0*RAD,         200000,              26400,           -80,         0},
  {                    754,    29.0*RAD,         180000,              23900,           -80,         0}, 
  {                    666,    25.0*RAD,         170000,              22700,           -80,         0},
  {                    592,     0.0*RAD,         160000,              22700,            40,         0},
  {                    567,     4.0*RAD,         150000,              22700,           -20,         0},
  {                    552,     4.0*RAD,         140000,              22400,           -28,         0},
  {                    414,    12.0*RAD,         120000,              21100,           -24,         0},
  {                    327,    13.0*RAD,         100000,              20600,           -25,         0},
  {                    275,    12.0*RAD,          80000,              18300,           -25,         0},
  {                    238,     9.0*RAD,          60000,              15000,           -25,         0},
  {                    221,     8.0*RAD,          50000,              13900,           -25,         0},
  {                    222,     6.0*RAD,          40000,              12600,           -35,         0},
  {                    238,     2.0*RAD,          30000,              10600,           -56,         0},
  {                    264,     1.0*RAD,          20000,               7000,           -80,         0},
  {                    204,     1.0*RAD,          10000,               2770,           -60,         0},
  {                    188,     1.0*RAD,           2000,                200,           -33,         0},
  {                    166,     2.0*RAD,              0,                  0,            -2,         0},
  {                    155,     0.0*RAD,          -1000,                  0,             0,         0},
  {                    116,     0.0*RAD,          -2000,                  0,             0,         0},
  {                      0,     0.0*RAD,          -3300,                  0,             0,         0}

	};

    G->RefSlopePts = 44;
    for (i=0;i<44;i++) {
	  for (j=0; j<6; j++) {
        G->RefSlope[i][j] = DefSlope[i][j];
      }
    }
	return;
}