//
// Glideslope Core Functions
//
// Purpose ... persistent class supporting the core calculations for GS2
// (The Glideslope class is very ephemeral ... it gets destroyed and reinstantiated each time you change view, etc, so persist the core into this class)
//
//
// (c) Andrew Stokes (ADSWNJ) 2012
//
// All rights reserved
//

#include "GlideslopeCore.h"
#define _CRT_SECURE_NO_WARNINGS

void GlideslopeCore::calcRunway() {
  llr_base=_V(RunwayData[runway][0],RunwayData[runway][1],0)*PI/180.0 +_V(0,0,re);
  xyz_base=llr_to_xyz(llr_base);
  nre_near_base=_V(-RunwayData[runway][2],re,RunwayData[runway][3]);
  nre_near_base=nre_near_base/length(nre_near_base)*re;
  nre_far_base=_V(-RunwayData[runway][4],re,RunwayData[runway][5]);
  nre_far_base=nre_far_base/length(nre_far_base)*re;
  rwy_length=length(nre_near_base-nre_far_base);
  M_base=m_nre_to_xyz(xyz_base);
  xyz_near=tmul(M_base,nre_near_base);
  xyz_far=tmul(M_base,nre_far_base);
  M_far=m_nre_to_xyz(xyz_far);
  nre_near_far=mul(M_far,xyz_near);
  ar_near_far=nre_to_az_range(nre_near_far,re);

  disp_papi=-RunwayData[runway][7];
  ar_papi_far=ar_near_far;
  ar_papi_far.range+=disp_papi;
  nre_papi_far=az_range_to_nre(ar_papi_far,re);
  xyz_papi=tmul(M_far,nre_papi_far);

  disp_tdp=-RunwayData[runway][6];
  ar_tdp_far=ar_near_far;
  ar_tdp_far.range+=disp_tdp;
  nre_tdp_far=az_range_to_nre(ar_tdp_far,re);
  xyz_tdp=tmul(M_far,nre_tdp_far);

  disp_wp2=FINAL_DIST;
  ar_wp2_far=ar_near_far;
  ar_wp2_far.range+=disp_wp2;
  nre_wp2_far=az_range_to_nre(ar_wp2_far,re);
  xyz_wp2=tmul(M_far,nre_wp2_far);

  base_radius_hac=HAC_RADIUS;
  radius_hac = base_radius_hac; 

  ra_wp2_far=ar_wp2_far.range/re;
  ra_hac=radius_hac/re;

  ra_hac_far=acos(cos(ra_wp2_far)*cos(ra_hac));
  ar_hac_far.range=ra_hac_far*re;
  daz_hac_far=acos(tan(ra_wp2_far)/tan(ra_hac_far));
  ar_hac_far.az=ar_wp2_far.az+daz_hac_far*hac_sign;
  nre_hac_far=az_range_to_nre(ar_hac_far,re);
  xyz_hac=tmul(M_far,nre_hac_far);
  M_hac=m_nre_to_xyz(xyz_hac);
  nre_wp2_hac=mul(M_hac,xyz_wp2);
  ar_wp2_hac=nre_to_az_range(nre_wp2_hac,re);

  rwy_pole=crossp(xyz_far,xyz_near);
  rwy_pole=rwy_pole/length(rwy_pole);

/*sprintf(oapiDebugString(),"Runway data Base(%s) Rwy(%s) LatLon(%4.1f %4.1f) Near(%4.1f %4.1f) Far(%4.1f %4.1f) TDP PAPI(%4.1f %4.1f)",
BaseName[RunwayBase[runway]], RunwayName[runway],
RunwayData[runway][0], RunwayData[runway][1], 
RunwayData[runway][2], RunwayData[runway][3], 
RunwayData[runway][4], RunwayData[runway][5], 
RunwayData[runway][6], RunwayData[runway][7] 
);
*/


}

void GlideslopeCore::calcRange() {
  //This part depends on the spacecraft position
  xyz_sc=llr_to_xyz(llr_sc);								// llr_sc is Lat/Long/Radius of spacecraft. Convert to xyz coords.
  M_sc=m_nre_to_xyz(xyz_sc);								// create spacecraft matrix
  nre_wp2_sc=mul(M_sc,xyz_wp2);								// Vector ship to WP2 (waypoint 2 = exit HAC into top of final descent)
  double scrossrange=dotp(rwy_pole,xyz_sc)/length(xyz_sc);	// Calculate scaled crossrange
  crossrange_angle=asin(scrossrange);						// Calculate crossrange angle
  crossrange=crossrange_angle*re;							// Calculate crossrange (perpendiculare left-right distance) to extended runway centerline
  nre_sc_far=mul(M_far,xyz_sc);								// Vector far end runway to ship
  nre_far_sc=mul(M_sc,xyz_far);								// Vector ship to far end runway 
  ar_sc_far=nre_to_az_range(nre_sc_far,re);					// Range far end runway to ship
  nre_near_sc=mul(M_sc,xyz_near);							// Vector ship to near end of runway
  nre_hac_sc=mul(M_sc,xyz_hac);								// Vector ship to the HAC

  // Assess leg 3 entry criteria
  if (( ( fabs(crossrange) < 2000) && (finalArmed || ( Altitude < 8000 ))) || (maxleg>2)) { // 
															// On final when sub 2km crossrange and final armed from leg2, or if we skipped the hac, sub 8km will do fine too
    leg=3;													// If we reach here, we are on leg 3, past the HAC on final approach
	maxleg=3;												// If on final, lock us on final
    VECTOR3 nre_tdp_sc=mul(M_sc,xyz_tdp);					// Vector from ship to touchdown point
    AZ_RANGE ar_tdp_sc=nre_to_az_range(nre_tdp_sc,re);		// Calc azimuth & range to touchdown point from relative vector
    az_tgt_sc=ar_tdp_sc.az;									// Azimuth to touchdown point
    range_papi_sc=ar_tdp_sc.range+disp_tdp-disp_papi;		// Range ship to the PAPI ... tdp less the VASI distance (start or rwy) less the PAPI (e.g. 20000 - 671 - 2000)
    range=ar_sc_far.range-rwy_length-disp_tdp;				// Actual range .. range to far end minus rwy length plus the VASI dist. (Disp TDP is negative, so this is correct!)
//sprintf(oapiDebugString(),"Leg 3 Range(FINAL)(%4.1f) Range(CRS, Angle deg)(%4.1f,%5.2f) Range(TDP, dtdp, dpapi, far, len)(%4.1f %4.1f %4.1f %4.1f %4.1f)", range, fabs(crossrange), crossrange_angle * DEG,
//	ar_tdp_sc.range, disp_tdp, disp_papi, ar_sc_far.range, rwy_length);
    return;
  }

  // Leg 1 or 2 common calcs
  range=ar_wp2_far.range-ar_tdp_far.range;					// For On-HAC or Approach HAC, start with the range from HAC exit to rwy touchdown point
  nre_sc_hac=mul(M_hac,xyz_sc);								// calculate HAC center vector to ship (multiply the xyz position by the hac matrix)
  ar_sc_hac=nre_to_az_range(nre_sc_hac,re);					// calculate the ship's range to the HAC center
    
  // Assess leg 2 entry criteria
  if((ar_sc_hac.range<(radius_hac*1.10)) || (maxleg>1)) {	// we assess on-HAC when we get within 10% of the hac radius on this approach (or we have been here already!)
    leg=2;													// If we reach here, we are on leg 2, past descent phase. 
	maxleg=2;												// If on HAC, lock out descent phase
    double az_wp2_hac=ar_wp2_hac.az*hac_sign;				// if HAC sign is inverted, we will go the opposite way round the HAC
    double az_sc_hac=ar_sc_hac.az*hac_sign;					// so do this consistently for ship's azimuth as well as WP2
    while (az_sc_hac< az_wp2_hac) az_sc_hac+=2*PI;			// make sure ship azimuth is higher (more radians) than WP2 by adding a circle's worth (2 PI) on 
    hac_arc=az_sc_hac-az_wp2_hac;							// HAC arc is now the radian segment round the HAC from ship to wp2
	if ((hac_arc > 1.5 * PI) && !WP2armed) {				// We enter the HAC with WP2 exit disarmed, so we make sure we do a lap ...
															//  ... without this, the arc range goes to zero as we roll round from WP1 to WP2 and then jumps up alarmingly by a full lap
		WP2armed = true;									//  ... so we check if we have gone past the WP2 point (hac_arc jumps from 0 to just under 2 * PI) and if so  arm WP2 exit
 	} else {												
		if (!WP2armed) hac_arc += 2*PI;						//  ... else if not, then we need to add on a lap's worth of arc. 
	}
    if ((hac_arc < 0.5 * PI) && WP2armed) {		 			// We arm the finals code block (entry to leg 3) when we are within a quarter turn of completion of the hac
		finalArmed = true;									//  ... and WP2 exit is armed.  
 	}

	arc_range=hac_arc*(radius_hac+ar_sc_hac.range)/2;		// Arc range is the hac_arc angle round the circle, times the mid-point of the reference hac radius and our actual radius 
    range+=arc_range;										// range starts with WP2 to TDP, now gets the arc range added
    VECTOR3 nre_tdp_sc=mul(M_sc,xyz_tdp);					// Vector ship to the tdp
    AZ_RANGE az_range_far_sc=nre_to_az_range(nre_tdp_sc,re); // Convert to azimuth and range
    range_papi_sc=range+disp_tdp-disp_papi;					// This PAPI range is correct ... range to tdp less dist_tdp (which is -ve) less dist papi (which is +ve) ... 10000 - 671 - 2000
    az_tgt_sc=az_sc_hac+PI/2*-hac_sign;						// Azimuth to target is a quarter turn round from current hac (for plotting track)
//sprintf(oapiDebugString(),"Leg 2 Range(HAC, HACdeg, FINAL)(%4.1f,%4.1f,%4.1f) RAD(HAC,CUR)(%4.1f, %4.1f) ARM(WP2,FIN)(%i,%i)", arc_range, hac_arc*DEG, ar_wp2_far.range-ar_tdp_far.range, radius_hac, ar_sc_hac.range, WP2armed?1:0, finalArmed?1:0);
    return;
  }

  // Fall through to leg 1
  leg=1;													//If we reach here, we are on leg 1, approaching the HAC
  maxleg=1;													// Lock us on descent phase for now
  
  double A=acos(tan(ra_hac)/tan(ar_sc_hac.range/re));
  double delta_az_wp1=acos(sin(A)*cos(ra_hac))*hac_sign;	// Calculate delta azimuth to WP1 (waypoint 1 - entry to the hac)
  double ra_wp1_sc=atan(sin(ra_hac)*tan(A));
  if(ra_wp1_sc<0) ra_wp1_sc=PI+ra_wp1_sc;
  if(fabs(DelAz)>PI/2) ra_wp1_sc=2*PI-ra_wp1_sc;
  range_wp1_sc=ra_wp1_sc*re;								// Calculate range from the ship to WP1
  nre_hac_sc=mul(M_sc,xyz_hac);								// Vector ship to the hac
  ar_hac_sc=nre_to_az_range(nre_hac_sc,re);					// Azimuth and range to the hac
  ar_wp1_sc.az=ar_hac_sc.az+delta_az_wp1;					// Azimuth to WP1 relative to our current bearing
  ar_wp1_sc.range=range_wp1_sc;								// Range is simply the range to WP1
  az_tgt_sc=ar_wp1_sc.az;									// Azimuth to target is the WP1 azimuth
  nre_wp1_sc=az_range_to_nre(ar_wp1_sc,re);					// Vector ship to WP1
  xyz_wp1=tmul(M_sc,nre_wp1_sc);							// Absolute coords of WP1
  nre_wp1_hac=mul(M_hac,xyz_wp1);							// Vector HAC center to WP1
  nre_wp1_far=mul(M_far,xyz_wp1);							// Vector far end of rwy to WP1
  ar_wp1_hac=nre_to_az_range(nre_wp1_hac,re);				// Azimuth & range HAC to WP1
  double az_wp2_hac=ar_wp2_hac.az*hac_sign;					// Azimuth of Waypoint 2 (in radians). This is exit point from the HAC. 
  double az_wp1_hac=ar_wp1_hac.az*hac_sign;					// Azimuth of Waypoint 1 (in radians). This is the entry point onto the HAC circle.
  while (az_wp1_hac< az_wp2_hac) az_wp1_hac+=2*PI;			// Make WP1 bigger in radians than WP2
  hac_arc=az_wp1_hac-az_wp2_hac;							// HAC arc is the arc segment distance from WP1 to WP2
  if (hac_open==0) {										// If the HAC is not open ... we do a lap round then the WP1-WP2 arc then exit, so
		hac_arc +=2*PI;										// Add on 2 PI to the HAC ARC
		WP2armed = false;									// And make sure the WP2 is not armed (keep HAC closed)
  } else {
	  WP2armed = true;										// HAC is open, so arm WP2
  }
  arc_range=hac_arc*radius_hac*1.05;						// ARC RANGE is the distance round the HAC. We do WP1 to WP2, then 2*PI (a lap) times the radius. (Leg 2 kicks in at 1.1x HAC, averaging to 1.05 radius)
  range+=arc_range;											// Add to the final range
  range+=range_wp1_sc;										// Add current distance to WP1
  range_papi_sc=range+disp_tdp-disp_papi;					// This PAPI range is correct ... range to tdp less dist_tdp (which is -ve) less dist papi (which is +ve) ... 10000 - 671 - 2000

//sprintf(oapiDebugString(),"Leg 1 Range(WP, HAC, HACdeg, FINAL)(%4.1f,%4.1f,%4.1f, %4.1f) Range(toHAC, L2trig)(%4.1f,%4.1f) RadHAC(BASE,ACT)(%4.1f,%4.1f) AZ1,2(%4.1f,%4.1f)", range_wp1_sc, arc_range, hac_arc*DEG, (ar_wp2_far.range-ar_tdp_far.range), ar_sc_hac.range, radius_hac*1.10,	1.0*base_radius_hac, 1.0*radius_hac, az_wp1_hac*DEG, az_wp2_hac*DEG);

}





void GlideslopeCore::MajorCycle(double simt) {
  vessel->GetEquPos(Lon,Lat,Altitude);
  llr_sc=_V(Lat,Lon,Altitude);
  Altitude-=re;
  vessel->GetHorizonAirspeedVector(AirspeedVec);
  vspd=AirspeedVec.y;
  Groundspeed=sqrt(AirspeedVec.x*AirspeedVec.x+AirspeedVec.z*AirspeedVec.z);
  Bearing=atan2(AirspeedVec.x,AirspeedVec.z);
  while(Bearing<0)Bearing+=2*PI;
  vessel->GetRelativeVel(Planet,InertialVel);
  InertialSpd=length(InertialVel);
  Mach=vessel->GetMachNumber();
  M2 = vessel->GetMass();

  TrimGain=linterp(8,1,2.5,0,Mach,false);
  if(TrimGain>1)TrimGain=1;
  if(TrimGain<0)TrimGain=0;
  //sprintf(oapiDebugString(),"Mach %f TrimGain %f",Mach,TrimGain);
  Airspeed=vessel->GetAirspeed();
  DynPres=vessel->GetDynPressure();
  HeatFlux=DynPres*Airspeed;
  calcRange();
  DelAz=az_tgt_sc-Bearing;
  while(DelAz<-PI)DelAz+=2*PI;
  while(DelAz>PI)DelAz-=2*PI;
  int RangeSeg;
  if (range>RefSlope[0][2]) {
    RangeSeg=-1;
  } else {
    for(RangeSeg=0;range<RefSlope[RangeSeg][2] && RangeSeg<RefSlopePts;RangeSeg++);
  }
  RangeSeg--;
  hasRefSlope=(RangeSeg>=0);
  if(hasRefSlope) {
    Refalpha=   linterp(RefSlope[RangeSeg+1][2],RefSlope[RangeSeg+1][1],RefSlope[RangeSeg][2],RefSlope[RangeSeg][1],range, false);
    RefAltitude=linterp(RefSlope[RangeSeg+1][2],RefSlope[RangeSeg+1][3],RefSlope[RangeSeg][2],RefSlope[RangeSeg][3],range, false);
    RefAirspeed=linterp(RefSlope[RangeSeg+1][2],RefSlope[RangeSeg+1][0],RefSlope[RangeSeg][2],RefSlope[RangeSeg][0],range, false);
  } else {
    RefVspd=0;
    Refalpha=40*RAD;
    RefAltitude=0;
    RefAirspeed=0;
  }
  PE=(GtimesM1 * M2 / (re)) - (GtimesM1 * M2 / (re + Altitude));
  KE=0.5 * M2 * Airspeed * Airspeed;
  TE=PE+KE;
  RefPE=(GtimesM1 * M2 / (re)) - (GtimesM1 * M2 / (re + RefAltitude));
  RefKE=0.5 * M2 * RefAirspeed * RefAirspeed;
  RefTE=RefPE+RefKE;

  AirspeedRate=(Airspeed-lastAirspeed)/(simt-lastSimT);
  RefAirspeedRate=(RefAirspeed-lastRefAirspeed)/(simt-lastSimT);
  RefVspd=(RefAltitude-lastRefAltitude)/(simt-lastSimT);
  RefVspdRate=(RefVspd-lastRefVspd)/(simt-lastSimT);
  RefTERate=(RefTE-lastRefTE)/(simt-lastSimT);
  BearingRate=(Bearing-lastBearing)/(simt-lastSimT);
  VspdRate=(vspd-lastVspd)/(simt-lastSimT);
  TERate=(TE-lastTE)/(simt-lastSimT);
  lastSimT=simt;
  lastTE=TE;
  lastRefTE=RefTE;
  lastVspd=vspd;
  lastRefVspd=RefVspd;
  lastBearing=Bearing;
  lastAirspeed=Airspeed;
  lastAltitude=Altitude;
  lastRefAirspeed=RefAirspeed;
  lastRefAltitude=RefAltitude;
  TrackT[TrackPtr]=simt;
  TrackD[TrackPtr]=range;
  TrackV[TrackPtr]=Airspeed;
  TrackA[TrackPtr]=Altitude;
  TrackVspd[TrackPtr]=vspd;
  TrackAlpha[TrackPtr]=alpha;
  TrackBeta[TrackPtr]=beta;
  TrackGamma[TrackPtr]=gamma;
  TrackRefAlpha[TrackPtr]=Refalpha;
  Trackvreldot[TrackPtr]=AirspeedRate;
  TrackRefvreldot[TrackPtr]=RefAirspeedRate;
  xyz_track[TrackPtr]=xyz_sc;
  TrackPtr++;
  if(TrackPtr>10000) TrackPtr=0;

/*  // On HAC approach, adjust Radius_HAC dynamically according to Alt error and TAS error
  if (leg==1) {
    radius_hac = base_radius_hac + ((2 * ((Airspeed-RefAirspeed)/RefAirspeed)) * base_radius_hac) + ((0.2 * ((Altitude-RefAltitude)/RefAltitude)) * base_radius_hac);
    if (radius_hac < base_radius_hac /2) radius_hac = base_radius_hac /2;

//sprintf(oapiDebugString(),"radius hac (%4.1f) = base(%4.1f) as(%4.1f) ref(%4.1f) factor(%6.3f)  alt(%4.1f) ref(%4.1f) factor(%6.3f)",
//radius_hac, base_radius_hac, Airspeed, RefAirspeed, (2 * ((Airspeed-RefAirspeed)/RefAirspeed)), 
//Altitude, RefAltitude, (0.2 * ((Altitude-RefAltitude)/RefAltitude)));

  }
*/
  
}

void GlideslopeCore::steer(double YawCmd, bool YawActive, double PitchCmd, bool PitchActive, double RollCmd, bool RollActive) {
  if(YawActive && steerActive) {
    if(YawCmd>0) {
      vessel->SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT,0);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT,YawCmd);
    } else {
      vessel->SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT,-YawCmd);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT,0);
    }
  }
  if(PitchActive && steerActive) {
    if(TrimGain>0.5) {
      double simt=oapiGetSimTime();
      double TrimSpd=PitchCmd*(simt-TrimLastSimT)*TrimGain;
      TrimPos+=PitchCmd*(simt-TrimLastSimT);
      //sprintf(oapiDebugString(),"TrimSpd: %f TrimPos: %f",TrimSpd,TrimPos);
      TrimLastSimT=simt;
      if(TrimPos>1.0)TrimPos=1.0;
      if(TrimPos<-1.0)TrimPos=-1.0;
      vessel->SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,TrimPos);
    }
    if(PitchCmd>0) {
      vessel->SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,PitchCmd);
    } else {
      vessel->SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,-PitchCmd);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,0);
    }
  }
  if(RollActive && steerActive) {
    if(RollCmd>0) {
      vessel->SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,RollCmd);
    } else {
      vessel->SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,-RollCmd);
      vessel->SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0);
    }
  }
}

void GlideslopeCore::MinorCycle(double simt) {
  alpha=vessel->GetAOA();
  beta=vessel->GetSlipAngle();
  gamma=vessel->GetBank();
  double alphaRate=(alpha-lastAlpha)/(simt-lastMinorSimT);
  double betaRate=(beta-lastBeta)/(simt-lastMinorSimT);
  double gammaRate=(gamma-lastGamma)/(simt-lastMinorSimT);
  lastAlpha=alpha;
  lastBeta=beta;
  lastGamma=gamma;
  lastMinorSimT=simt;
  if (simt>nextMajorCycle) {
    MajorCycle(simt);
    nextMajorCycle=simt+1;
  }
  double YawCmd=0;
  double PitchCmd=0;
  double RollCmd=0;
  double TrimCmd=0;
  double KPYaw=-5;
  double KDYaw=-25;
  double KPPitch=-5;
  double KDPitch=-25;
  double KPRoll=0;
  double KDRoll=5;
  if(Altitude>BETA_GUIDE_OFF_ALT) {
    double PYaw=beta;
    double DYaw=betaRate;
    YawCmd=KPYaw*PYaw+KDYaw*DYaw;
    double PPitch=alpha-Refalpha;
    double DPitch=alphaRate;
    PitchCmd=KPPitch*PPitch+KDPitch*DPitch;
    double PRoll=0;
    double DRoll=gammaRate;
    RollCmd=KPRoll*PRoll+KDRoll*DRoll;
    double ManualRoll=vessel->GetManualControlLevel(THGROUP_ATT_BANKRIGHT,MANCTRL_ANYDEVICE,MANCTRL_ROTMODE)-vessel->GetManualControlLevel(THGROUP_ATT_BANKLEFT,MANCTRL_ANYDEVICE,MANCTRL_ROTMODE);
    if(fabs(ManualRoll)>0.1)RollCmd=ManualRoll;
  }
  steer(YawCmd,Altitude>BETA_GUIDE_OFF_ALT,PitchCmd,Altitude>BETA_GUIDE_OFF_ALT,RollCmd,Altitude>BETA_GUIDE_OFF_ALT);
}

void GlideslopeCore::Save(bool units) {
  errno_t err;
  FILE* ouf;
  
  if ((err = fopen_s(&ouf, ".\\Config\\MFD\\GS2\\GS2glideslope_UserSave.cfg","w")) != 0) {
	return;
  };
  double FindRangeM[50] = { 8155, 8005, 7505, 7005, 6505, 6005, 5505, 5005, 4505, 4005,
	                        3505, 3005, 2505, 2005, 1505, 1005,  905,  805,  705,  605,
							 505,  455,  405,  355,  305,  275,  255,  225,  205,  195,
							 185,  175,  165,  155,  145,  135,  125,  105,   92,   82,
							  72,   62,   52,   42,   32,   22,   11,    2,   -1,   -2};
  int fr=0, maxfr=50, i=1;


  if (ouf!=NULL) {
	fprintf(ouf,";\n");
	fprintf(ouf,"; Glideslope 2 User-Saved Glideslope Data\n");
	fprintf(ouf,";\n");
	fprintf(ouf,"UNITS %s\n",(units?"US":"METRIC"));
	fprintf(ouf,"BEGIN GLIDESLOPE\n");
	fprintf(ouf,  ";   Range       Alt          TAS       VSpd        AoA\n");
	if (!units) {
  	  fprintf(ouf,";   km          km           m/s       m/s          \n");
	} else {
	  fprintf(ouf,";   NMi         kft          ft/s      ft/s         \n");
	}
	fprintf(ouf,  ";    ------      -----      -----     -----        -----\n");

	// Find starting range
	while ((fr<maxfr)&&(FindRangeM[fr]*1000>TrackD[1])) {
//fprintf(ouf,"; FSR fr %i FindRangeM[fr]*1000 %10.1f, TrackD[1] %10.1f\n", fr,FindRangeM[fr]*1000,TrackD[1]);
 	  fr++;};

	// Dump each line immediately below range
	do {
		while ((TrackD[i]>FindRangeM[fr]*1000) && (i<TrackPtr)) {
//fprintf(ouf,"; DEL i %i TrackD[i]  %10.2f >FindRangeM[fr]*1000 %10.2f\n", i, TrackD[i], FindRangeM[fr]*1000);
			
			i++;};
		if (i<TrackPtr) {
			if (!units) {
			  fprintf(ouf,"%10.2f %10.2f %10.0f %10.1f %10.1f   ; Time %i secs\n",
			    TrackD[i]/1000,TrackA[i]/1000,TrackV[i],TrackVspd[i],TrackAlpha[i]*DEG, (int) TrackT[i]);
			} else {
			  fprintf(ouf,"%10.2f %10.2f,%10.0f %10.1f %10.1f   ; Time %i secs\n",
			    TrackD[i]/1852,TrackA[i]/304.8,TrackV[i]/0.3048,TrackVspd[i]/0.3048,TrackAlpha[i]*DEG, (int) TrackT[i]);
			}
			fr++; i++;
		}
    } while ((fr<maxfr)&&(i<TrackPtr));
	fprintf(ouf,"END GLIDESLOPE\n");

	// Debug full dump
//	for (i=0;i<TrackPtr;i++) {
//  	  fprintf(ouf,";     RNG %10.2f ALT %10.2f TAS %10.0f VSP %10.1f AOA %10.1f   ; Time %i secs, Row %i\n",
//	    TrackD[i],TrackA[i],TrackV[i],TrackVspd[i],TrackAlpha[i]*DEG, (int) TrackT[i], i);
//	}

    fclose(ouf);
  }
}

void GlideslopeCore::reset(VESSEL *v) {
  vessel=v;
  runway=0;
  TrackPtr=0;
  hac_sign=1;
  hac_open=0;
  steerActive=false;
  isInit=true;
  Planet=vessel->GetGravityRef();
  re=oapiGetSize(Planet);
  GtimesM1=GGRAV*oapiGetMass(Planet);
  runway = ParamsRwy;
  calcRunway();
  DeorbitHp=-25000;
  WP2armed = false;
  finalArmed = false;
  maxleg = 1;

}

