// Test.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#define _CRT_SECURE_NO_WARNINGS
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

void LoadConfig ();
void LoadDefaultConfig ();

int _tmain(int argc, _TCHAR* argv[])
{

	printf("Hello world\n");

	LoadConfig();



	return 0;
}

// Load these up from GS2.cfg
const int RunwayLimit=128;
const int BaseLimit=64;
int RunwayCount;
int BaseCount;
double RunwayData[RunwayLimit][9];
int RunwayBase[RunwayLimit];
char RunwayName[RunwayLimit][12];
char BaseName[BaseLimit][32];
double BaseLL[BaseLimit][2];
int ParamsMode, ParamsRwy;
bool ParamsUnits;

// String parser helpers
// bp is buffer pointer ... will be updated to point after the parsed element
// ret is the returned item
// functions return true if the parse was good, else false
// note .. the buffer contents are changed (nulls inserted to tokenize)
  bool ParseQuotedString(char **bp, char **ret) {
	  char *b = *bp;
	  while ((*b==' ')||(*b=='\t')) b++;
	  if (*b!='\"') return false;
	  b++;
	  *ret = b;
	  while ((*b!='\"')&&(*b!='\0')&&(*b!='\n')) b++;
	  if (*b!='\"') return false;
	  *b = '\0';
	  *bp = b+1;
	  return true;
  }

  bool ParseString(char **bp, char **ret) {

	  char *b = *bp;
	  while ((*b==' ')||(*b=='\t')) b++;
	  *ret = b;
	  while ((*b!='\"')&&(*b!='\0')&&(*b!='\n')&&(*b!=' ')&&(*b!='\'')&&(*b!='\t')) b++;
	  if ((*b=='\"')||(*b=='\'')) return false; // No quotes allowed in string
	  if (*b!='\0') {
		*b = '\0';
		*bp=b+1;
	  } else {
		*bp = b;	// if we hit EOL, point to EOL not one beyond
	  }
	  return true;
  }

  bool ParseDouble(char **bp, double *ret) {
	  char *b = *bp;
	  char *t;
	  int i;

	  while ((*b==' ')||(*b=='\t')) b++;
	  t = b;
	  i = strspn(t, "+-0123456789e.");
	  b = t+i;
	  if ((*b!=' ')&&(*b!='\t')&&(*b!='\n')&&(*b!='\0')) return false; // End of parse must be whitespace
	  if (*b!='\0') {
		*b = '\0';
		*bp=b+1;
	  } else {
		*bp = b;	// if we hit EOL, point to EOL not one beyond
	  }
	  if (i==0) return false;
	  *ret = atof(t);
	  return true;
  }
  bool ParseInt(char **bp, int *ret) {
	  double f=0.0;
	  if (!ParseDouble(bp,&f)) return false;
	  *ret=int(f);
	  return true;
  }

  bool ParseBool(char **bp, bool *ret) {
	  char *bufp;
	  if (!ParseString(bp,&bufp)) return false;
	  if (stricmp(bufp,"FALSE")==0) {
		  *ret = false;
		  return true;
	  }
	  if (stricmp(bufp,"TRUE")==0) {
		  *ret = true;
		  return true;
	  }
	  return false;
  }


void LoadConfig () {
  FILE* GS2cfg;
//  int scancount;
  char buf[256];
  char bufcpy[256];
  buf[255] = '\0';
  char *tok=buf;
  char *strtok=buf;
  double tokf[6];
  int j;
  int ofs;
  int modetok = 0;
  bool unitstok = false;

  char *bp;
  bool prefsFound = false, runwayFound = false, baseFound = false; 

//  GS2cfg = fopen(".\\Config\\MFD\\GS2\\GS2.cfg","r");
  GS2cfg = fopen("J:\\Orbiter\\Config\\MFD\\GS2\\GS2.cfg","r");
  if (GS2cfg == NULL) {
// Defaults to init code
	  LoadDefaultConfig();
	  return;
  }

  BaseCount = 0;
  RunwayCount = 0;

//  FILE* GS2CfgPL=fopen(".\\Config\\MFD\\GS2\\GS2cfgParselog.txt","w");
  FILE* GS2CfgPL=fopen("J:\\Orbiter\\Config\\MFD\\GS2\\GS2cfgParselog.txt","w");
  fprintf(GS2CfgPL,"Successfully found and opened GS2.cfg\n");

  while (fgets(buf,255,GS2cfg)!=NULL) {
	bp=buf;
	sprintf(bufcpy,"%s",buf);
	while (bp[0]==' ' || bp[0]=='\t') bp++;
	if ((bp[0]=='\0')||(bp[0]=='\n')) {
      fprintf(GS2CfgPL,"NUL> %s", buf);
	  continue;
	}
	if (bp[0]==';') {
      fprintf(GS2CfgPL,"REM> %s", buf);
	  continue;
	}

	if (!ParseString(&bp,&tok)) goto badparse;

	if (stricmp(tok,"BASE")==0)
	{
// Parsing BASE "base name" lon lat
// e.g.    BASE "Cape Canaveral" -80.675 +28.5208

	  if (!ParseQuotedString(&bp,&strtok)) goto badparse;
	  if (strlen(strtok)>32) goto badparse;
	  for (j=0; j<2; j++) {
	    if (!ParseDouble(&bp,&(tokf[j]))) goto badparse;
	  }
	  if (BaseCount >= BaseLimit) goto badparse;
	  sprintf(BaseName[BaseCount],strtok);
	  for (j=0; j<2; j++) BaseLL[BaseCount][j] = tokf[j];
	  fprintf(GS2CfgPL,"BASE LOADED: %s Lon:%.3f Lat:%.3f\n", BaseName[BaseCount],BaseLL[BaseCount][0],BaseLL[BaseCount][0] );
	  baseFound = true;
	  BaseCount++;
	  continue;
	}

	if (stricmp(tok,"RUNWAY")==0)
	{
// Parsing RUNWAY "{basename}" "{rwyname}" {end1-1} {end1-3} {end2-1} {end2-3} {papi} {vasi}
// e.g.    RUNWAY "Cape Canaveral" "KSC33" -8220 -600 -12670 -3155 -2000 671

	  if (!ParseQuotedString(&bp,&strtok)) goto badparse;
	  
	  for (ofs=0; (ofs<BaseCount) && (stricmp(strtok,(BaseName[ofs]))!=0) ; ofs++) {}
	  if (ofs==BaseCount) goto badparse;

	  if (!ParseQuotedString(&bp,&strtok))  goto badparse;
	  if (strlen(strtok)>12) goto badparse;

	  for (j=0; j<6; j++) {
	    if (!ParseDouble(&bp,&(tokf[j]))) goto badparse;
	  }
	  if (RunwayCount >= RunwayLimit) goto badparse;

	  sprintf(RunwayName[RunwayCount],strtok);
	  for (j=0;j<2;j++) RunwayData[RunwayCount][j] = BaseLL[ofs][j];
	  for (j=0;j<6;j++) RunwayData[RunwayCount][j+2] = tokf[j];

	  fprintf(GS2CfgPL,"RWY LOADED: %s\'s %s END1(%.0f, %.0f) END2(%.0f, %.0f) PAPI=%.0f VASI=%.0f\n", BaseName[ofs], RunwayName[RunwayCount], 
		  RunwayData[RunwayCount][2], RunwayData[RunwayCount][3], RunwayData[RunwayCount][4],
		  RunwayData[RunwayCount][5], RunwayData[RunwayCount][6], RunwayData[RunwayCount][7]);
	  runwayFound = true;
	  RunwayCount++;
	  continue;
	}

	if (stricmp(tok,"PREFS")==0)
	{

// Parsing PREFS {mode} {USunits} {rwy}   {glideslope}
// e.g.    PREFS 2      false     "KSC33" XR

	  if (prefsFound) goto badparse;

	  if (!ParseInt(&bp,&modetok)) goto badparse;
	  if (!ParseBool(&bp,&unitstok))  goto badparse;
	  if (!ParseQuotedString(&bp,&strtok)) goto badparse;
	  for (ofs=0; (ofs<RunwayCount) && (stricmp(strtok,(RunwayName[ofs]))!=0) ; ofs++) {}
	  if (ofs==RunwayCount) goto badparse;
	  if (!ParseString(&bp,&tok)) goto badparse;

	  ParamsMode = modetok;
	  ParamsUnits = unitstok;
	  ParamsRwy = ofs;
	  
	  fprintf(GS2CfgPL,"PREFS LOADED: Mode=%i Units=%s Runway=%s Glideslope=%s\n", ParamsMode, (ParamsUnits?"US":"METRIC"), RunwayName[ParamsRwy], tok );
	  BaseCount++;
	  prefsFound = true;
	  continue;
		
   }
   badparse:
   fprintf(GS2CfgPL,"Bad parse, ignored line>> %s", bufcpy);
   fprintf(GS2CfgPL,"(Check for typos, base not previously defined, base>32 chars, runway >12 chars, >64 bases, > 128 rwys, >1 prefs line, etc.)\n\n");
  }

  if (!feof(GS2cfg)) {
    fprintf(GS2CfgPL,"Bad read on GS2.cfg\n");
    LoadDefaultConfig();
  } else {
    if ((!prefsFound)||(!baseFound)||(!runwayFound)) {
      if (!prefsFound) fprintf(GS2CfgPL,"No PREFS found ... defaulting to Default Config");
      if (!baseFound) fprintf(GS2CfgPL,"No BASE found ... defaulting to Default Config");
      if (!runwayFound) fprintf(GS2CfgPL,"No RUNWAY found ... defaulting to Default Config");
      LoadDefaultConfig();
    } else {
      fprintf(GS2CfgPL,"Successfully parsed GS2.cfg\n");
	}
  }
  fclose(GS2CfgPL);
  fclose(GS2cfg);

  return;
}


void LoadDefaultConfig () {return;}
