// Test.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#define _CRT_SECURE_NO_WARNINGS
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

void LoadPreferredGlideslopeConfig (char *PrefFile);
void LoadDefaultGlideslopeConfig ();

int _tmain(int argc, _TCHAR* argv[])
{

	printf("Test rig for glideslope\n");

	LoadPreferredGlideslopeConfig("Test");
//	LoadPreferredGlideslopeConfig("XR_200_120_75_Dot7");



	return 0;
}

// Load these up from GS2.cfg
const double DTOR=3.1415926/180.0;
const int GlideslopePointsLimit=256;
int RefSlopePts;
double RefSlope[GlideslopePointsLimit][6];

// String parser helpers
// bp is buffer pointer ... will be updated to point after the parsed element
// ret is the returned item
// functions return true if the parse was good, else false
// note .. the buffer contents are changed (nulls inserted to tokenize)
  bool ParseQuotedString(char **bp, char **ret) {
	  char *b = *bp;
	  while ((*b==' ')||(*b=='\t')) b++;
	  if (*b!='\"') return false;
	  b++;
	  *ret = b;
	  while ((*b!='\"')&&(*b!='\0')&&(*b!='\n')) b++;
	  if (*b!='\"') return false;
	  *b = '\0';
	  *bp = b+1;
	  return true;
  }

  bool ParseString(char **bp, char **ret) {

	  char *b = *bp;
	  while ((*b==' ')||(*b=='\t')) b++;
	  *ret = b;
	  while ((*b!='\"')&&(*b!='\0')&&(*b!='\n')&&(*b!=' ')&&(*b!='\'')&&(*b!='\t')) b++;
	  if ((*b=='\"')||(*b=='\'')) return false; // No quotes allowed in string
	  if (*b!='\0') {
		*b = '\0';
		*bp=b+1;
	  } else {
		*bp = b;	// if we hit EOL, point to EOL not one beyond
	  }
	  return true;
  }

  bool ParseDouble(char **bp, double *ret) {
	  char *b = *bp;
	  char *t;
	  int i;

	  while ((*b==' ')||(*b=='\t')) b++;
	  t = b;
	  i = strspn(t, "+-0123456789e.");
	  b = t+i;
	  if ((*b!=' ')&&(*b!='\t')&&(*b!='\n')&&(*b!='\0')) return false; // End of parse must be whitespace
	  if (*b!='\0') {
		*b = '\0';
		*bp=b+1;
	  } else {
		*bp = b;	// if we hit EOL, point to EOL not one beyond
	  }
	  if (i==0) return false;
	  *ret = atof(t);
	  return true;
  }
  bool ParseInt(char **bp, int *ret) {
	  double f=0.0;
	  if (!ParseDouble(bp,&f)) return false;
	  *ret=int(f);
	  return true;
  }

  bool ParseBool(char **bp, bool *ret) {
	  char *bufp;
	  if (!ParseString(bp,&bufp)) return false;
	  if (stricmp(bufp,"FALSE")==0) {
		  *ret = false;
		  return true;
	  }
	  if (stricmp(bufp,"TRUE")==0) {
		  *ret = true;
		  return true;
	  }
	  return false;
  }

 
void LoadPreferredGlideslopeConfig (char *PrefFile) {
  FILE* GS2gscfg;
  char buf[256];
  char bufcpy[256];
  buf[255] = '\0';
  char *tok=buf;
  char *strtok=buf;
  int i, j;
  int modetok = 0;
  bool unitstok = false;

  double gsparams[GlideslopePointsLimit][6];
  int gsrow=0;

  char *bp;
  bool unitsFound = false, gsFound = false, inGSblock = false;
  bool unitMode = false;

  sprintf(buf,".\\Config\\MFD\\GS2\\GS2glideslope_%s.cfg",PrefFile);

  GS2gscfg = fopen(buf,"r");
  if (GS2gscfg == NULL) {
// Defaults to init code
	  LoadDefaultGlideslopeConfig();
	  return;
  }
  sprintf(buf,".\\Config\\MFD\\GS2\\GS2glideslope_%s_ParseLog.txt",PrefFile);

  FILE* GS2CfgPL=fopen(buf,"w");
  fprintf(GS2CfgPL,"Successfully found and opened %s\n",buf);

  while (fgets(buf,255,GS2gscfg)!=NULL) {
	bp=buf;
	sprintf(bufcpy,"%s",buf);
	while (bp[0]==' ' || bp[0]=='\t') bp++;
	if ((bp[0]=='\0')||(bp[0]=='\n')) {
      fprintf(GS2CfgPL,"NUL> %s", buf);
	  continue;
	}
	if (bp[0]==';') {
      fprintf(GS2CfgPL,"REM> %s", buf);
	  continue;
	}

	if (!ParseString(&bp,&tok)) goto badparse;

	if (_stricmp(tok,"UNITS")==0)
	{
// Parsing UNITS METRIC or UNITS US

	  if (!ParseString(&bp,&strtok)) goto badparse;
  	  if (_stricmp(strtok,"METRIC")==0) {
		  unitsFound = true;
		  unitMode = false;
          fprintf(GS2CfgPL,"UNITS are Metric\n");
	      continue;
	  } else if (_stricmp(strtok,"US")==0) {
		  unitsFound = true;
		  unitMode = true;
          fprintf(GS2CfgPL,"UNITS are US\n");
		  continue;
	  } else goto badparse;
	}

	if (_stricmp(tok,"BEGIN")==0)
	{
// Parsing BEGIN GLIDESLOPE

	  if (!ParseString(&bp,&strtok)) goto badparse;
  	  if (_stricmp(strtok,"GLIDESLOPE")!=0) goto badparse;
	  if (!unitsFound) goto badparse;
	  if (inGSblock) goto badparse;
	  inGSblock = true;
	  continue;
	}

	if (_stricmp(tok,"END")==0)
	{
// Parsing END GLIDESLOPE

	  if (!ParseString(&bp,&strtok)) goto badparse;
  	  if (_stricmp(strtok,"GLIDESLOPE")!=0) goto badparse;
	  if (!inGSblock) goto badparse;
	  inGSblock = false;
	  continue;
	}

// To get here .... we should be in the GS details 
    if (!inGSblock) goto badparse;
    if (gsrow>GlideslopePointsLimit) goto badparse;

	gsparams[gsrow][0] = atoi(tok);
	for (j=1;j<5;j++) {
      if (!ParseDouble(&bp,&(gsparams[gsrow][j]))) goto badparse;
	}
	gsparams[gsrow][5] = 0;
	gsrow++;
    fprintf(GS2CfgPL,"Loaded glideslope point> %s", bufcpy);
    continue;

badparse:
   fprintf(GS2CfgPL,"Bad parse, ignored line>> %s", bufcpy);
   fprintf(GS2CfgPL,"(Check for typos, no UNITS line, badly formed GLIDESLOPE data etc.)\n\n");
  }

  if (!feof(GS2gscfg)) {
    fprintf(GS2CfgPL,"Bad read on GS2 glideslope points file\n");
    LoadDefaultGlideslopeConfig();
  } else {
    if (gsrow<2) {
      fprintf(GS2CfgPL,"Need glideslope points in a glideslope points file ... defaulting to Default Config");
      LoadDefaultGlideslopeConfig();
    } else {
      fprintf(GS2CfgPL,"Successfully parsed GS2 glideslope points file\n");
	}
  }

// Transfer points into master structure
  const int pmap[6] = {2, 3, 0, 4, 1, 5};		// Order from points file different to order in the internal table. Fix it up
  const char colname[6][4] = {"RNG","ALT","TAS","VSP","AOA","ZRO"};
  const double cnvM[6] = {1000, 1000, 1, 1, DTOR, 0};	// Conversion for metric points entry Rng, Alt, TAS, Vspd, AOA, 0
  const double cnvU[6] = {1852, 304.8, 0.3048, 0.3048, DTOR, 0};	// Conversion for US points entry Rng, Alt, TAS, Vspd, AOA, 0
  double cnv[6];

  for (j=0;j<6;j++) {cnv[j] = (unitMode? cnvU[j] : cnvM[j]);};

  for (i=0;i<3;i++) {
      fprintf(GS2CfgPL,"Gsparams Dump:\n ");
	  for (j=0; j<6; j++) {
		  fprintf(GS2CfgPL, "GSparams[%i][%i]=%f\n",i,j,gsparams[i][j]);
      }
  }

  RefSlopePts = gsrow;
  for (i=0;i<gsrow;i++) {
      fprintf(GS2CfgPL,"GS2 point loaded: ");
	  for (j=0; j<6; j++) {
        RefSlope[i][pmap[j]] = gsparams[i][j]*cnv[j]; 
        fprintf(GS2CfgPL,"%s %.3f (%.3f * %.3f) ",colname[j], RefSlope[i][pmap[j]], gsparams[i][j], cnv[j] );
      }
      fprintf(GS2CfgPL,"\n");
  }
  fprintf(GS2CfgPL,"Completed\n");

   for (i=0;i<3;i++) {
      fprintf(GS2CfgPL,"RefSlope dump:\n ");
	  for (j=0; j<6; j++) {
		  fprintf(GS2CfgPL, "Refslope[%i][%i]=%f\n",i,j,RefSlope[i][j]);
      }
  }
 

  fclose(GS2CfgPL);
  fclose(GS2gscfg);

  return;
}

void LoadDefaultGlideslopeConfig () {

    int i, j;
	const double DefSlope[44][6]={


  // New Ref Slope in Meters
  //           TAS m/s         AoA deg        Range, m              Alt, m      dAlt, m/s         Segment length
  //-----------------------------------------------------------
 
  {                   7501,    41.0*DTOR,        8145000,             120000,           -95,         0},
  {                   7504,    41.0*DTOR,        8000000,             118000,           -95,         0},
  {                   7520,    41.5*DTOR,        7000000,             105000,           -94,         0}, 
  {                   7533,    41.5*DTOR,        6000000,              93000,           -89,         0},
  {                   7534,    42.5*DTOR,        5500000,              87000,           -85,         0},
  {                   7524,    42.5*DTOR,        5000000,              81500,           -79,         0},
  {                   7491,    42.0*DTOR,        4500000,              76500,           -70,         0},
  {                   7416,    42.0*DTOR,        4000000,              72000,           -60,         0},
  {                   7285,    42.0*DTOR,        3500000,              68500,           -55,         0},
  {                   7078,    43.0*DTOR,        3000000,              65500,           -55,         0}, 
  {                   6774,    43.0*DTOR,        2500000,              62000,           -55,         0},
  {                   6245,    40.5*DTOR,        2000000,              56000,           -55,         0},
  {                   5352,    40.5*DTOR,        1500000,              50500,           -55,         0},
  {                   4176,    36.5*DTOR,        1000000,              46500,           -50,         0},
  {                   3652,    36.5*DTOR,         800000,              44600,           -50,         0},
  {                   2987,    35.5*DTOR,         600000,              41000,           -50,         0},
  {                   2567,    34.0*DTOR,         500000,              39000,           -50,         0},
  {                   2294,    33.0*DTOR,         450000,              38500,           -50,         0},
  {                   2109,    33.0*DTOR,         400000,              37600,           -60,         0}, 
  {                   1908,    33.0*DTOR,         350000,              35700,           -90,         0},
  {                   1645,    33.0*DTOR,         300000,              33100,           -90,         0},
  {                   1491,    33.0*DTOR,         275000,              31800,           -80,         0},
  {                   1320,    33.0*DTOR,         250000,              30400,           -70,         0},
  {                   1135,    33.0*DTOR,         225000,              28900,           -80,         0},
  {                    919,    33.0*DTOR,         200000,              26400,           -80,         0},
  {                    754,    29.0*DTOR,         180000,              23900,           -80,         0}, 
  {                    666,    25.0*DTOR,         170000,              22700,           -80,         0},
  {                    592,     0.0*DTOR,         160000,              22700,            40,         0},
  {                    567,     4.0*DTOR,         150000,              22700,           -20,         0},
  {                    552,     4.0*DTOR,         140000,              22400,           -28,         0},
  {                    414,    12.0*DTOR,         120000,              21100,           -24,         0},
  {                    327,    13.0*DTOR,         100000,              20600,           -25,         0},
  {                    275,    12.0*DTOR,          80000,              18300,           -25,         0},
  {                    238,     9.0*DTOR,          60000,              15000,           -25,         0},
  {                    221,     8.0*DTOR,          50000,              13900,           -25,         0},
  {                    222,     6.0*DTOR,          40000,              12600,           -35,         0},
  {                    238,     2.0*DTOR,          30000,              10600,           -56,         0},
  {                    264,     1.0*DTOR,          20000,               7000,           -80,         0},
  {                    204,     1.0*DTOR,          10000,               2770,           -60,         0},
  {                    188,     1.0*DTOR,           2000,                200,           -33,         0},
  {                    166,     2.0*DTOR,              0,                  0,            -2,         0},
  {                    155,     0.0*DTOR,          -1000,                  0,             0,         0},
  {                    116,     0.0*DTOR,          -2000,                  0,             0,         0},
  {                      0,     0.0*DTOR,          -3300,                  0,             0,         0}

	};

    RefSlopePts = 44;
    for (i=0;i<43;i++) {
	  for (j=0; j<6; j++) {
        RefSlope[i][j] = DefSlope[i][j];
      }
    }
	return;
}